@@ -70,12 +70,6 @@ struct acldata {
     char *object;
 };
 
-extern struct sasl_client krb_sasl_client;
-struct sasl_client *login_sasl_client[] = {
-    &krb_sasl_client,
-    NULL
-};
-
 struct cbdata cb;
 struct acldata acldata;
 int content_mode, acl_mode, aclro_mode, verbose, debug, noncommit;
@@ -170,7 +164,7 @@ Usage: syncams [-A | -a] [-d] [-h] [-m]\n\
     exit(1);
 } /* usage */
 
-void fatal(char *msg, int exitvalue)
+void fatal(const char *msg, int exitvalue)
 {
     fputs("amssync: ",stderr);
     fputs(msg,stderr);
@@ -641,6 +635,9 @@ do_content(char *amsname, char *imapname)
 void cyr_connect(void)
 {
     int code, errs;
+    int minssf=0;
+    int maxssf=0;
+    char *user = 0;
 
     if (verbose) {
 	fprintf(logfile,"Connecting to Cyrus server...\n");
@@ -667,9 +664,11 @@ void cyr_connect(void)
 	    fprintf(stderr,"Unknown error %d from imclient_connect\n",code);
 	    exit(1);
 	}
-	code = imclient_connect(&imclient,server,port);
+	code = imclient_connect(&imclient, server, port);
     }
-    if (imclient_authenticate(imclient,login_sasl_client,"imap",NULL,SASL_PROT_ANY)) {
+    /* we have AMS, so we have AFS, so we have Kerberos */
+    if (imclient_authenticate(imclient, "KERBEROS_V4", "imap",
+			      user, minssf, maxssf)) {
 	fprintf(stderr,"Couldn't authenticate to server!\n");
 	exit(1);
     }
@@ -805,7 +804,7 @@ Port: %s\n\
 			    BBHASH_SIZE /* size; a magic number */,
 			    sizeof(char*), /* size of member (useless,
 					      really */
-			    strcmp, /* compare fn */
+			    (int(*)(void*,void*))strcmp, /* compare fn */
 			    free /* free fn */);
 	
 	/* tjs: add list callback */
@@ -854,7 +853,9 @@ Port: %s\n\
 			  "LIST \"\" %s.*", rexp+(*rexp=='^'));
 	    cbwait();
 
-	    if (debug) { ht_foreach(bb_hash, (void*) puts); }
+#if 0
+	    if (debug) { ht_foreach(bb_hash, ((void*) puts); }
+#endif
 	}
 
 	sprintf(submap,"%s/.MESSAGES/.SubscriptionMap",dir);
@@ -902,14 +903,14 @@ Port: %s\n\
 
     if (debug && acl_mode) {
 	puts("The following bboards remain in hash table:");
-	ht_foreach(bb_hash, (void*) puts);
+	ht_foreach(bb_hash, (void(*)(void*)) puts);
     }
 
     /* tjs */
     /* for any bboard left in the hash table,
        blast it. */
     if (acl_mode) {
-	ht_foreach(bb_hash, (void*) bbdelete);
+	ht_foreach(bb_hash, (void(*)(void*)) bbdelete);
     }
 
     do_imap_noop(imclient);	/* Flush & wait for pending commands */
