@@ -28,25 +28,22 @@
 #include <errno.h>
 #include <string.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <pwd.h>
+#include <sys/types.h>
+
+#include <sasl.h>
+
 extern int errno;
 
-#include "sasl.h"
 #include "imclient.h"
 #include "imparse.h"
 #include "tcl.h"
 #include "xmalloc.h"
 
-#ifdef HAVE_SASL_KRB
-extern struct sasl_client krb_sasl_client;
-#endif
-
-struct sasl_client *login_sasl_client[] = {
-#ifdef HAVE_SASL_KRB
-    &krb_sasl_client,
-#endif
-    NULL
-};
-
 struct admconn {
     struct imclient *imclient;
     int cmd_done;
@@ -361,6 +358,125 @@ char **argv;
     return conn->cmd_result;
 }
 
+/***********************
+ * Parse a mech list of the form: ... AUTH=foo AUTH=bar ...
+ *
+ * Return: string with mechs seperated by spaces
+ *
+ ***********************/
+
+typedef struct capabilies_s {
+
+  char *mechs;
+  
+  /* 0 = false; 1 = true */
+  int starttls;
+  int logindisabled;
+
+} capabilities_t;
+
+
+
+static capabilities_t *parsecapabilitylist(char *str)
+{
+  char *tmp;
+  int num=0;
+  capabilities_t *ret=(capabilities_t *) xmalloc(sizeof(capabilities_t));
+  ret->mechs = (char *)xmalloc(strlen(str)+1);
+  ret->starttls=0;
+  ret->logindisabled=0;
+
+  /* check for stattls */
+  if (strstr(str,"STARTTLS")!=NULL)
+  {
+    ret->starttls=1;
+  }
+
+  /* check for login being disabled */
+  if (strstr(str,"LOGINDISABLED")!=NULL)
+  {
+    ret->logindisabled=1;
+  }
+
+  strcpy(ret->mechs,"");
+
+  while ((tmp=strstr(str,"AUTH="))!=NULL)
+  {
+    char *end=tmp+5;
+    tmp+=5;
+
+    while(((*end)!=' ') && ((*end)!='\0'))
+      end++;
+
+    (*end)='\0';
+
+    /* add entry to list */
+    if (num>0)
+      strcat(ret->mechs," ");
+    strcat(ret->mechs, tmp);
+    num++;
+
+    /* reset the string */
+    str=end+1;
+
+  }
+
+  return ret;
+}
+
+/*
+ * IMAP command completion callback
+ */
+static void callback_capability(struct imclient *imclient, 
+				void *rock,
+				struct imclient_reply *reply)
+
+{
+    char *s;
+    capabilities_t **caps = (capabilities_t **) rock;
+    
+    s = reply->text;
+ 
+    *caps = parsecapabilitylist(s);
+}
+
+/*
+ * Use the IMAP login command
+ */
+
+static int cmd_login(struct admconn *conn, char *userid, char *pass, int passlen, int tls_layer, int logindisabled)
+{
+  char **comv;
+
+  if (logindisabled==1)
+  {
+    printf("Login Disabled. Aborting\n");
+    return -1;
+  }
+
+  if (pass==NULL)
+  {
+    /*    if (tls_layer==0)
+	  printf("Warning: About to send password cleartext (ctrl-c to abort)\n"); */
+
+    printf("Password: ");
+    pass = getpass("");
+    passlen = strlen(pass);
+  }
+
+
+  imclient_send(conn->imclient, callback_finish, (void *)conn,
+		"LOGIN %s {%d+}\r\n%s", 
+		userid, passlen, pass);
+
+  conn->cmd_done = 0;
+  while (!conn->cmd_done) {
+    imclient_processoneevent(conn->imclient);
+  }
+
+  return conn->cmd_result;  
+}
+
 /*
  * Perform the authenticate subcommand
  */
@@ -373,10 +489,15 @@ char **argv;
 {
     char *pwcommand = 0;
     char *user = 0;
-    int prot = SASL_PROT_ANY;
     char *p;
     int r;
-    
+    int minssf=0;     /* default to allow any security layer */
+    int maxssf=10000;
+    char *mech = NULL;
+    char *tls_keyfile = "";
+    capabilities_t *capabilitylist;
+    int tls_layer = 0;
+
     /* skip over command & subcommand */
     argv += 2;
 
@@ -390,46 +511,94 @@ char **argv;
 	    if (!argv[1]) break;
 	    user = *++argv;
 	}
-	else if (!strcmp(argv[0], "-protection")) {
+	else if (!strcmp(argv[0], "-layers")) {
 	    if (!argv[1]) break;
-	    prot = 0;
-	    for (p = *++argv; *p; p++) {
-		switch (*p) {
-		case 'n':
-		    prot |= SASL_PROT_NONE;
-		    break;
-
-		case 'i':
-		    prot |= SASL_PROT_INTEGRITY;
-		    break;
-
-		case 'p':
-		    prot |= SASL_PROT_PRIVACY;
-		    break;
-
-		default:
-		    Tcl_AppendResult(interp, "incorrect protection mode: ",
-				     "should be one or more of n, i, p",
-				     (char *) NULL);
-		    return TCL_ERROR;
-		}
-	    }
+	    maxssf = atoi(*++argv);
+	}
+	else if (!strcmp(argv[0], "-mech")) {
+	    if (!argv[1]) break;
+	    mech = *++argv;
+	} else if (!strcmp(argv[0], "-tlskey")) {
+	    if (!argv[1]) break;
+	    tls_keyfile = *++argv;
+	} else if (!strcmp(argv[0], "-notls")) {
+	    tls_keyfile = NULL;
 	}
 	argv++;
     }
     if (*argv) {
+#ifdef HAVE_SSL
 	Tcl_AppendResult(interp, "incorrect args: should be \"",
 			 argv[0], " authenticate ",
 			 "[-pwcommand string] [-user user] ",
-			 "[-protection (n|i|p)+\"",
+			 "[-layers #] [-mech mechname]\"",
 			 (char *) NULL);
+#else /* HAVE_SSL */
+	Tcl_AppendResult(interp, "incorrect args: should be \"",
+			 argv[0], " authenticate ",
+			 "[-pwcommand string] [-user user] ",
+			 "[-layers #] [-mech mechname] [-tlskey keyfile] [-notls]\"",
+			 (char *) NULL);
+#endif /* HAVE_SSL */
+
 	return TCL_ERROR;
     }
 
-    r = imclient_authenticate(conn->imclient, login_sasl_client, "imap",
-			      user, prot);
+    if (!user) {
+	user = xmalloc(sizeof(char) * 1024);
+	strcpy(user, getpwuid(getuid())->pw_name);
+    }
+
+    imclient_addcallback(conn->imclient, "CAPABILITY", 0,
+			 callback_capability, (void *) &capabilitylist, 
+			 (char *) 0);
+
+    imclient_send(conn->imclient, callback_finish, (void *) conn,
+		  "CAPABILITY");
+
+    while (!conn->cmd_done) {
+	imclient_processoneevent(conn->imclient);
+    }
+
+#ifdef HAVE_SSL
+    /* starttls unless user told us not to */
+    if (capabilitylist->starttls == 1)
+    {
+      if (tls_keyfile!=NULL)
+      {
+	imclient_starttls(conn->imclient,
+			  10,
+			  tls_keyfile, tls_keyfile,
+			  &tls_layer);
+
+	/* ask for capabilities again */
+
+	imclient_addcallback(conn->imclient, "CAPABILITY", 0,
+			     callback_capability, (void *) &capabilitylist, 
+			     (char *) 0);
+
+	imclient_send(conn->imclient, callback_finish, (void *) conn,
+		      "CAPABILITY");
+
+	conn->cmd_done = 0;	
+	while (!conn->cmd_done) {
+	  imclient_processoneevent(conn->imclient);
+	}
+
+      }
+    }
+#endif /* HAVE_SSL */
+
+    if (!pwcommand) {
+	r = imclient_authenticate(conn->imclient, mech ? mech : capabilitylist->mechs, 
+				  "imap", user, minssf, maxssf);
+
+	if (r == SASL_NOMECH) {
+	    r = cmd_login(conn, user, NULL, 0, tls_layer, capabilitylist->logindisabled);
+	}
+    }
     
-    if (r == 1 && (prot & SASL_PROT_NONE) && pwcommand) {
+    if (pwcommand) {
 	Tcl_DString command;
 	int comc;
 	char **comv;
@@ -478,14 +647,7 @@ char **argv;
 	    return TCL_ERROR;
 	}
 
-	imclient_send(conn->imclient, callback_finish, (void *)conn,
-		      "LOGIN %s %s", comv[0], comv[1]);
-	free((char *)comv);
-
-	while (!conn->cmd_done) {
-	    imclient_processoneevent(conn->imclient);
-	}
-	r = conn->cmd_result;
+	r = cmd_login(conn, comv[0], comv[1], strlen(comv[1]), tls_layer, capabilitylist->logindisabled);
     }
 
     if (r) {
