@@ -1,5 +1,5 @@
 /* append.h -- Description of messages to be copied 
- $Id: append.h,v 1.12 1998/05/15 21:48:03 neplokh Exp $
+ $Id: append.h,v 1.13 1999/12/30 21:07:25 leg Exp $
  
  # Copyright 1998 Carnegie Mellon University
  # 
@@ -32,14 +32,6 @@
 
 #include "prot.h"
 
-#ifndef P
-#ifdef __STDC__
-#define P(x) x
-#else
-#define P(x) ()
-#endif
-#endif
-
 struct copymsg {
     unsigned long uid;
     time_t internaldate;
@@ -53,22 +45,35 @@ struct copymsg {
     char *flag[MAX_USER_FLAGS+1];
 };
 
-extern int append_setup P((struct mailbox *mailbox, const char *name,
-			   int format, struct auth_state *auth_state,
-			   long aclcheck, long quotacheck));
+extern int append_setup(struct mailbox *mailbox, const char *name,
+			int format, struct auth_state *auth_state,
+			long aclcheck, long quotacheck);
 
-extern int append_fromstream P((struct mailbox *mailbox,
-				struct protstream *messagefile,
-				unsigned long size, time_t internaldate,
-				const char **flag, int nflags,
-				const char *userid));
+struct stagemsg;
 
-extern int append_copy P((struct mailbox *mailbox,
-			  struct mailbox *append_mailbox,
-			  int nummsg, struct copymsg *copymsg,
-			  const char *userid));
+/* adds a new mailbox to the stage. creates the stage if *stagep == NULL */
+extern int append_fromstage(struct mailbox *mailbox,
+			    struct protstream *messagefile,
+			    unsigned long size, time_t internaldate,
+			    const char **flag, int nflags,
+			    const char *userid,
+			    struct stagemsg **stagep);
 
-extern int append_collectnews P((struct mailbox *mailbox,
-				 const char *group, unsigned long feeduid));
+/* removes the stage (frees memory, deletes the staging files) */
+extern int append_removestage(struct stagemsg *stage);
+
+extern int append_fromstream(struct mailbox *mailbox,
+			     struct protstream *messagefile,
+			     unsigned long size, time_t internaldate,
+			     const char **flag, int nflags,
+			     const char *userid);
+
+extern int append_copy(struct mailbox *mailbox,
+		       struct mailbox *append_mailbox,
+		       int nummsg, struct copymsg *copymsg,
+		       const char *userid);
+
+extern int append_collectnews(struct mailbox *mailbox,
+			      const char *group, unsigned long feeduid);
 
 #endif /* INCLUDED_APPEND_H */
