@@ -26,7 +26,7 @@
  *
  */
 
-/* $Id: arbitron.c,v 1.13 1998/11/24 22:25:36 shadow Exp $ */
+/* $Id: arbitron.c,v 1.16 1999/08/31 18:39:19 leg Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -39,7 +39,7 @@
 
 #include "assert.h"
 #include "config.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
@@ -72,7 +72,7 @@ char **argv;
 
     strcpy(pattern, "*");
 
-    if (geteuid() == 0) fatal("must run as the Cyrus user", EX_USAGE);
+    if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
     while ((opt = getopt(argc, argv, "d:p:")) != EOF) {
 	switch (opt) {
@@ -91,7 +91,7 @@ char **argv;
 	}
     }
 
-    if (optind != argc) strncpy(pattern, argv[1], MAX_MAILBOX_NAME);
+    if (optind != argc) strncpy(pattern, argv[optind], MAX_MAILBOX_NAME);
 
     report_time = time(0) - (report_days*60*60*24);
     if (prune_months) {
@@ -106,7 +106,7 @@ char **argv;
 usage()
 {
     fprintf(stderr, "usage: arbitron [-d days] [-p months] [mboxpattern]\n");
-    exit(EX_USAGE);
+    exit(EC_USAGE);
 }    
 
 int
@@ -185,9 +185,7 @@ char *name;
     return r;
 }
 
-fatal(s, code)
-char *s;
-int code;
+void fatal(const char* s, int code)
 {
     fprintf(stderr, "arbitron: %s\n", s);
     exit(code);
