@@ -1,5 +1,5 @@
 /* collectnews.c -- program to add news articles to relevant header files
- $Id: collectnews.c,v 1.14 1998/05/15 21:48:11 neplokh Exp $
+ $Id: collectnews.c,v 1.19 1999/10/02 00:43:03 leg Exp $
  
  # Copyright 1998 Carnegie Mellon University
  # 
@@ -31,9 +31,12 @@
 #include <string.h>
 #include <syslog.h>
 #include <com_err.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include "config.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
@@ -45,6 +48,8 @@ struct newsgroup {
     char groupname[1];
 };
 
+void collect(char *group, unsigned long feeduid);
+
 struct newsgroup **newsgroup = 0;
 int num_newsgroup = 0;
 int size_newsgroup = 0;
@@ -54,23 +59,21 @@ const char *newsprefix;
 int newsprefixlen;
 
 
-main(argc, argv)
-int argc;
-char **argv;
+int main(int argc, char **argv)
 {
     char buf[4096], *group, *nextgroup, *uid, *p;
     int c;
 
     config_init("collectnews");
 
-    if (geteuid() == 0) fatal("must run as the Cyrus user", EX_USAGE);
+    if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
     newsprefix = config_getstring("newsprefix", 0);
     if (newsprefix) newsprefixlen = strlen(newsprefix);
     
     if (!config_getstring("partition-news", 0)) {
 	fatal("partition-news option not specified in configuration file",
-	      EX_CONFIG);
+	      EC_CONFIG);
     }
 
     while (fgets(buf, sizeof(buf), stdin)) {
@@ -97,6 +100,7 @@ char **argv;
 	    }
 	}
 
+#ifdef OLD_INN_FORMAT
 	/* Nuke out overview junk */
 	if (p = strchr(buf, '\t')) {
 	    *p = '\0';
@@ -117,15 +121,30 @@ char **argv;
 
 	    collect(group, atol(uid));
 	} while ((group = nextgroup) != 0);
+#else
+	/* this is the new INN format that uses the replication data */
+
+	/* we see "group.foo.A/#,group.bar.B/#,group.baz.C/#" */
+	group = buf;
+	while (group) {
+	    nextgroup = strchr(group, ',');
+	    if (nextgroup) *nextgroup++ = '\0';
+
+	    uid = strrchr(group, '/');
+	    if (!uid) continue;
+	    *uid++ = '\0';
+	    
+	    collect(group, atol(uid));
+	    
+	    group = nextgroup;
+	}
     }
+#endif
     exit(0);
 }
 
-collect(group, feeduid)
-char *group;
-unsigned long feeduid;
+void collect(char *group, unsigned long feeduid)
 {
-    
     int r;
     struct mailbox mailbox;
     char namebuf[MAX_MAILBOX_PATH];
@@ -248,30 +267,28 @@ int r;
 	return 0;
 	
     case IMAP_IOERROR:
-	return EX_IOERR;
+	return EC_IOERR;
 
     case IMAP_PERMISSION_DENIED:
-	return EX_NOPERM;
+	return EC_NOPERM;
 
     case IMAP_QUOTA_EXCEEDED:
-	return EX_TEMPFAIL;
+	return EC_TEMPFAIL;
 
     case IMAP_MAILBOX_NOTSUPPORTED:
-	return EX_DATAERR;
+	return EC_DATAERR;
 
     case IMAP_MAILBOX_NONEXISTENT:
-	return EX_UNAVAILABLE;
+	return EC_UNAVAILABLE;
     }
 	
     /* Some error we're not expecting. */
-    return EX_SOFTWARE;
+    return EC_SOFTWARE;
 }	
 
-fatal(s, code)
-char *s;
-int code;
+void fatal(const char* s, int code)
 {
-    fprintf(stderr, "collectnews: %s\n", s);
+    syslog(LOG_CRIT, "collectnews: %s\n", s);
     exit(code);
 }
 
