@@ -1,5 +1,5 @@
 /* config.c -- Configuration routines
- $Id: config.c,v 1.17 1998/05/15 21:48:14 neplokh Exp $
+ $Id: config.c,v 1.22 1999/10/02 00:43:03 leg Exp $
  
  # Copyright 1998 Carnegie Mellon University
  # 
@@ -27,29 +27,19 @@
  *
  */
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <ctype.h>
 #include <syslog.h>
 #include <com_err.h>
 
 #include "config.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "xmalloc.h"
 
 extern int errno;
 
 #define CONFIG_FILENAME "/etc/imapd.conf"
-/* You'd think this'd be EX_CONFIG, but you'd be wrong.
-   If it's EX_CONFIG then sendmail's result is to fatally reject the
-   message; this isn't desireable, we just want to fail the message in a
-   more limited case.
-
-   We had a situation where someone changed imapd.conf and made it
-   unreadable by Cyrus.  In 13 seconds, 4 messages were rejected, so I
-   changed the code.
-   
-   XXX hopefully sendmail is the only meaningful place where this matters.
-   */
-#define CONFIG_EXIT_STATUS EX_TEMPFAIL
 
 struct configlist {
     char *key;
@@ -63,9 +53,11 @@ const char *config_dir;
 const char *config_defpartition;
 const char *config_newsspool;
 
+int config_hashimapspool;
+
 static void config_read P((void));
 
-config_init(ident)
+int config_init(ident)
 const char *ident;
 {
     char buf[100];
@@ -83,7 +75,7 @@ const char *ident;
     config_dir = config_getstring("configdirectory", (char *)0);
     if (!config_dir) {
 	fatal("configdirectory option not specified in configuration file",
-	      CONFIG_EXIT_STATUS);
+	      EC_CONFIG);
     }
 
     mboxlist_checkconfig();
@@ -93,13 +85,13 @@ const char *ident;
     for (p = (char *)config_defpartition; *p; p++) {
 	if (!isalnum(*p))
 	  fatal("defaultpartition option contains non-alphanumeric character",
-		CONFIG_EXIT_STATUS);
+		EC_CONFIG);
 	if (isupper(*p)) *p = tolower(*p);
     }
     if (!config_partitiondir(config_defpartition)) {
 	sprintf(buf, "partition-%s option not specified in configuration file",
 		config_defpartition);
-	fatal(buf, CONFIG_EXIT_STATUS);
+	fatal(buf, EC_CONFIG);
     }
 
     /* Look up umask */
@@ -113,6 +105,9 @@ const char *ident;
     /* Look up news spool */
     config_newsspool = config_getstring("newsspool", 0);
 
+    /* look up mailbox hashing */
+    config_hashimapspool = config_getswitch("hashimapspool", 0);
+
     return 0;
 }
 
@@ -130,7 +125,7 @@ const char *def;
     return def;
 }
 
-config_getint(key, def)
+int config_getint(key, def)
 const char *key;
 int def;
 {
@@ -141,7 +136,7 @@ int def;
     return atoi(val);
 }
 
-config_getswitch(key, def)
+int config_getswitch(key, def)
 const char *key;
 int def;
 {
@@ -186,7 +181,7 @@ config_read()
     if (!infile) {
 	sprintf(buf, "can't open configuration file %s: %s", CONFIG_FILENAME,
 		error_message(errno));
-	fatal(buf, CONFIG_EXIT_STATUS);
+	fatal(buf, EC_CONFIG);
     }
     
     while (fgets(buf, sizeof(buf), infile)) {
@@ -197,7 +192,7 @@ config_read()
 	if (!*p || *p == '#') continue;
 
 	key = p;
-	while (*p && (isalnum(*p) || *p == '-')) {
+	while (*p && (isalnum(*p) || *p == '-' || *p == '_')) {
 	    if (isupper(*p)) *p = tolower(*p);
 	    p++;
 	}
@@ -205,7 +200,7 @@ config_read()
 	    sprintf(buf,
 		    "invalid option name on line %d of configuration file",
 		    lineno);
-	    fatal(buf, CONFIG_EXIT_STATUS);
+	    fatal(buf, EC_CONFIG);
 	}
 	*p++ = '\0';
 
@@ -214,7 +209,7 @@ config_read()
 	if (!*p) {
 	    sprintf(buf, "empty option value on line %d of configuration file",
 		    lineno);
-	    fatal(buf, CONFIG_EXIT_STATUS);
+	    fatal(buf, EC_CONFIG);
 	}
 
 	if (nconfiglist == alloced) {
