@@ -26,7 +26,7 @@
  *
  */
 /*
- * $Id: index.c,v 1.81 1998/10/29 20:16:29 tjs Exp $
+ * $Id: index.c,v 1.91 2000/01/04 19:42:23 leg Exp $
  */
 #include <stdio.h>
 #include <string.h>
@@ -43,7 +43,7 @@
 #include "util.h"
 #include "map.h"
 #include "assert.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "gmtoff.h"
 #include "imap_err.h"
 #include "mailbox.h"
@@ -82,17 +82,17 @@ struct seen *seendb;		/* Seen state database object */
 static char *seenuids;		/* Sequence of UID's from last seen checkpoint */
 
 /* Access macros for the memory-mapped index file data */
-#define INDEX_OFFSET(msgno) (index_base+start_offset+(((msgno)-1)*record_size))
-#define UID(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_UID)))
-#define INTERNALDATE(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_INTERNALDATE)))
-#define SENTDATE(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_SENTDATE)))
-#define SIZE(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_SIZE)))
-#define HEADER_SIZE(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_HEADER_SIZE)))
-#define CONTENT_OFFSET(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_CONTENT_OFFSET)))
-#define CACHE_OFFSET(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_CACHE_OFFSET)))
-#define LAST_UPDATED(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_LAST_UPDATED)))
-#define SYSTEM_FLAGS(msgno) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_SYSTEM_FLAGS)))
-#define USER_FLAGS(msgno,i) ntohl(*((bit32 *)(INDEX_OFFSET(msgno)+OFFSET_USER_FLAGS+((i)*4))))
+#define INDEC_OFFSET(msgno) (index_base+start_offset+(((msgno)-1)*record_size))
+#define UID(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_UID)))
+#define INTERNALDATE(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_INTERNALDATE)))
+#define SENTDATE(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_SENTDATE)))
+#define SIZE(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_SIZE)))
+#define HEADER_SIZE(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_HEADER_SIZE)))
+#define CONTENT_OFFSET(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_CONTENT_OFFSET)))
+#define CACHE_OFFSET(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_CACHE_OFFSET)))
+#define LAST_UPDATED(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_LAST_UPDATED)))
+#define SYSTEM_FLAGS(msgno) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_SYSTEM_FLAGS)))
+#define USER_FLAGS(msgno,i) ntohl(*((bit32 *)(INDEC_OFFSET(msgno)+OFFSET_USER_FLAGS+((i)*4))))
 
 /* Access assistance macros for memory-mapped cache file data */
 #define CACHE_ITEM_BIT32(ptr) (ntohl(*((bit32 *)(ptr))))
@@ -114,9 +114,10 @@ struct mapfile {
 typedef int index_sequenceproc_t P((struct mailbox *mailbox, unsigned msgno,
 				    void *rock));
 
-static int index_forsequence P((struct mailbox *mailbox, char *sequence,
-				int usinguid,
-				index_sequenceproc_t *proc, void *rock));
+static int index_forsequence(struct mailbox *mailbox, char *sequence,
+			     int usinguid,
+			     index_sequenceproc_t *proc, void *rock,
+			     int* fetchedsomething);
 static int index_insequence P((int num, char *sequence, int usinguid));
 
 static void index_fetchmsg P((const char *msg_base, unsigned long msg_size,
@@ -234,7 +235,7 @@ int checkseen;
 	}
 	else if (sbuf.st_ino != mailbox->index_ino) {
 	    if (mailbox_open_index(mailbox)) {
-		fatal("failed to reopen index file", EX_IOERR);
+		fatal("failed to reopen index file", EC_IOERR);
 	    }
 
 	    for (oldmsgno = msgno = 1; oldmsgno <= imapd_exists;
@@ -288,7 +289,7 @@ int checkseen;
     if (fstat(mailbox->cache_fd, &sbuf) == -1) {
 	syslog(LOG_ERR, "IOERROR: stating cache file for %s: %m",
 	       mailbox->name);
-	fatal("failed to stat cache file", EX_IOERR);
+	fatal("failed to stat cache file", EC_IOERR);
     }
     if (cache_end < sbuf.st_size) {
 	cache_end = sbuf.st_size;
@@ -300,6 +301,7 @@ int checkseen;
     if (oldexists == -1 && keepingseen) {
 	r = seen_open(mailbox, imapd_userid, &seendb);
 	if (!r) {
+	    free(seenuids);
 	    r = seen_lockread(seendb, &last_read, &recentuid,
 			      &seen_last_change, &seenuids);
 	    if (r) seen_close(seendb);
@@ -357,6 +359,8 @@ int checkseen;
 	}
         prot_printf(imapd_out, "* OK [UIDVALIDITY %u] \r\n",
 		    mailbox->uidvalidity);
+	prot_printf(imapd_out, "* OK [UIDNEXT %u] \r\n",
+		    mailbox->last_uid + 1);
     }
 
     for (msgno = 1; msgno <= oldexists; msgno++) {
@@ -663,16 +667,20 @@ int oldexists;
 
 /*
  * Perform a FETCH-related command on a sequence.
+ * Fetchedsomething argument is 0 if nothing was fetched, 1 if something was
+ * fetched.  (A fetch command that fetches nothing is not a valid fetch
+ * command.)
  */
 void
-index_fetch(mailbox, sequence, usinguid, fetchargs)
-struct mailbox *mailbox;
-char *sequence;
-int usinguid;
-struct fetchargs *fetchargs;
+index_fetch(struct mailbox* mailbox,
+	    char* sequence,
+	    int usinguid,
+	    struct fetchargs* fetchargs,
+	    int* fetchedsomething)
 {
+    *fetchedsomething = 0;
     index_forsequence(mailbox, sequence, usinguid,
-		      index_fetchreply, (char *)fetchargs);
+		      index_fetchreply, (char *)fetchargs, fetchedsomething);
 }
 
 /*
@@ -700,7 +708,7 @@ int nflags;
 	storeargs->usinguid = usinguid;
 
 	index_forsequence(mailbox, sequence, usinguid,
-			  index_storeseen, (char *)storeargs);
+			  index_storeseen, (char *)storeargs, NULL);
 	return 0;
     }
 
@@ -814,7 +822,7 @@ int nflags;
     if (r) return r;
 
     r = index_forsequence(mailbox, sequence, usinguid,
-			  index_storeflag, (char *)storeargs);
+			  index_storeflag, (char *)storeargs, NULL);
 
     mailbox_unlock_index(mailbox);
 
@@ -877,7 +885,7 @@ char **copyuidp;
 
     copyargs.nummsg = 0;
     index_forsequence(mailbox, sequence, usinguid, index_copysetup,
-		      (char *)&copyargs);
+		      (char *)&copyargs, NULL);
 
     if (copyargs.nummsg == 0) {
 	*copyuidp = 0;
@@ -1078,6 +1086,8 @@ struct mailbox *mailbox;
     return 0;
 }
 
+#if 0
+/* What's this for?  Might as well keep it around. */
 /*
  * Performs a XCHECKSTATE command
  */
@@ -1108,7 +1118,7 @@ unsigned seendate;
 	    prot_printf(imapd_out,
 			"* XCHECKSTATE %u:%u\r\n", UID(1), UID(imapd_exists));
 	}
-	return;
+	return 0;
     }
 
     prot_printf(imapd_out, "* XCHECKSTATE");
@@ -1139,6 +1149,7 @@ unsigned seendate;
     prot_printf(imapd_out, "\r\n");
     return 0;
 }
+#endif
 
 /*
  * Returns the msgno of the message with UID 'uid'.
@@ -1194,14 +1205,14 @@ char *indexbuf;
  * returns the first such returned value.  Otherwise, returns zero.
  */
 static int
-index_forsequence(mailbox, sequence, usinguid, proc, rock)
-struct mailbox *mailbox;
-char *sequence;
-int usinguid;
-index_sequenceproc_t *proc;
-void *rock;
+index_forsequence(struct mailbox* mailbox,
+		  char* sequence,
+		  int usinguid,
+		  index_sequenceproc_t proc,
+		  void* rock,
+		  int* fetchedsomething)
 {
-    int i, start = 0, end;
+    unsigned i, start = 0, end;
     int r, result = 0;
 
     /* no messages, no calls.  dumps core otherwise */
@@ -1240,6 +1251,7 @@ void *rock;
 	    if (start < 1) start = 1;
 	    if (end > imapd_exists) end = imapd_exists;
 	    for (i = start; i <= end; i++) {
+		if (fetchedsomething) *fetchedsomething = 1;
 		r = (*proc)(mailbox, i, rock);
 		if (r && !result) result = r;
 	    }
@@ -1253,6 +1265,7 @@ void *rock;
 		start = i;
 	    }
 	    if (start > 0 && start <= imapd_exists) {
+		if (fetchedsomething) *fetchedsomething = 1;
 		r = (*proc)(mailbox, start, rock);
 		if (r && !result) result = r;
 	    }
@@ -1272,7 +1285,7 @@ int num;
 char *sequence;
 int usinguid;
 {
-    int i, start = 0, end;
+    unsigned i, start = 0, end;
 
     for (;;) {
 	if (isdigit(*sequence)) {
@@ -2047,9 +2060,10 @@ void *rock;
 	    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 	char datebuf[30];
 
-	if (tm->tm_year < 80) {
+	if (msgdate == 0 || tm->tm_year < 69) {
 	    abort();
 	}
+
 	if (gmtoff < 0) {
 	    gmtoff = -gmtoff;
 	    gmtnegative = 1;
@@ -2235,16 +2249,15 @@ void *rock;
     int mid;
     int r;
     int firsttry = 1;
-    int seendirty = 0, dirty = 0;
+    int dirty = 0;
 
     /* Change \Seen flag */
-    if (storeargs->operation == STORE_REPLACE && (mailbox->myrights&ACL_SEEN)) {
-	if (seenflag[msgno] != storeargs->seen) seendirty++;
+    if (storeargs->operation == STORE_REPLACE && (mailbox->myrights&ACL_SEEN))
+    {
 	seenflag[msgno] = storeargs->seen;
     }
     else if (storeargs->seen) {
 	i = (storeargs->operation == STORE_ADD) ? 1 : 0;
-	if (seenflag[msgno] != i) seendirty++;
 	seenflag[msgno] = i;
     }
 
@@ -2326,35 +2339,34 @@ void *rock;
 	}
     }
 
-    if (dirty || seendirty) {
-	if (dirty) {
-	    /* If .SILENT, assume client has updated their cache */
-	    if (storeargs->silent && flagreport[msgno] &&
-		flagreport[msgno] == record.last_updated) {
-		flagreport[msgno] = 
-		  (record.last_updated >= storeargs->update_time) ?
-		    record.last_updated + 1 : storeargs->update_time;
-	    }
-
-	    record.last_updated =
-	      (record.last_updated >= storeargs->update_time) ?
+    if (dirty) {
+	/* If .SILENT, assume client has updated their cache */
+	if (storeargs->silent && flagreport[msgno] &&
+	    flagreport[msgno] == record.last_updated) {
+	    flagreport[msgno] = 
+		(record.last_updated >= storeargs->update_time) ?
 		record.last_updated + 1 : storeargs->update_time;
 	}
-
-	if (!storeargs->silent) {
-	    index_fetchflags(mailbox, msgno, record.system_flags,
-			     record.user_flags, record.last_updated);
-	    if (storeargs->usinguid) {
-		prot_printf(imapd_out, " UID %u", UID(msgno));
-	    }
-	    prot_printf(imapd_out, ")\r\n");
-	}
-
-	if (dirty && mid) {
-	    r = mailbox_write_index_record(mailbox, mid, &record);
-	    if (r) return r;
-	}
+	
+	record.last_updated =
+	    (record.last_updated >= storeargs->update_time) ?
+	    record.last_updated + 1 : storeargs->update_time;
     }
+    
+    if (!storeargs->silent) {
+	index_fetchflags(mailbox, msgno, record.system_flags,
+			 record.user_flags, record.last_updated);
+	if (storeargs->usinguid) {
+	    prot_printf(imapd_out, " UID %u", UID(msgno));
+	}
+	prot_printf(imapd_out, ")\r\n");
+    }
+    
+    if (dirty && mid) {
+	r = mailbox_write_index_record(mailbox, mid, &record);
+	if (r) return r;
+    }
+    
     return 0;
 }
 
@@ -2520,7 +2532,7 @@ const char *cacheitem;
     int partsleft = 1;
     int subparts;
     int start, len, charset, encoding;
-    char *p;
+    char *p, *q;
     
     /* Won't find anything in a truncated file */
     if (msgfile->size == 0) return 0;
@@ -2541,10 +2553,12 @@ const char *cacheitem;
 		    p = index_readheader(msgfile->base, msgfile->size,
 					 format, CACHE_ITEM_BIT32(cacheitem),
 					 len);
-		    p = charset_decode1522(p);
-		    if (charset_searchstring(substr, pat, p, strlen(p))) {
+		    q = charset_decode1522(p, NULL, 0);
+		    if (charset_searchstring(substr, pat, q, strlen(q))) {
+			free(q);
 			return 1;
 		    }
+		    free(q);
 		}
 	    }
 	    cacheitem += 5*4;
@@ -2582,15 +2596,18 @@ struct mapfile *msgfile;
 int format;
 int size;
 {
-    char *p;
+    char *p, *q;
+    int r;
     static struct strlist header;
 
     header.s = name;
 
     p = index_readheader(msgfile->base, msgfile->size, format, 0, size);
     index_pruneheader(p, &header, 0);
-    p = charset_decode1522(p);
-    return charset_searchstring(substr, pat, p, strlen(p));
+    q = charset_decode1522(p, NULL, 0);
+    r = charset_searchstring(substr, pat, q, strlen(q));
+    free(q);
+    return r;
 }
 
 /*
@@ -2603,12 +2620,13 @@ char *name;
 char *substr;
 comp_pat *pat;
 {
-    char *p;
+    char *p, *q;
     static struct strlist header;
     static char *buf;
     static int bufsize;
     const char *cacheitem;
     unsigned size;
+    int r;
 
     cacheitem = cache_base + CACHE_OFFSET(msgno);
     cacheitem = CACHE_ITEM_NEXT(cacheitem); /* skip envelope */
@@ -2631,8 +2649,10 @@ comp_pat *pat;
     index_pruneheader(buf, &header, 0);
     if (!*buf) return 0;	/* Header not present, fail */
     if (!*substr) return 1;	/* Only checking existence, succeed */
-    p = charset_decode1522(buf);
-    return charset_searchstring(substr, pat, p, strlen(p));
+    p = charset_decode1522(buf, NULL, 0);
+    r = charset_searchstring(substr, pat, q, strlen(q));
+    free(q);
+    return r;
 }
 
 /*
