@@ -1,5 +1,5 @@
 /* mailbox.c -- Mailbox manipulation routines
- $Id: mailbox.c,v 1.73 1998/06/25 22:16:02 tjs Exp $
+ $Id: mailbox.c,v 1.81 1999/10/02 00:43:04 leg Exp $
  
  # Copyright 1998 Carnegie Mellon University
  # 
@@ -31,6 +31,7 @@
 #include <unistd.h>
 #endif
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 #include <syslog.h>
@@ -38,6 +39,8 @@
 #include <fcntl.h>
 #include <netinet/in.h>
 #include <sys/stat.h>
+#include <ctype.h>
+#include <time.h>
 
 #if HAVE_DIRENT_H
 # include <dirent.h>
@@ -63,7 +66,7 @@
 #include "retry.h"
 #include "util.h"
 #include "lock.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "xmalloc.h"
@@ -157,7 +160,7 @@ unsigned long *lenp;
     
     if (fstat(msgfd, &sbuf) == -1) {
 	syslog(LOG_ERR, "IOERROR: fstat on %s: %m", buf);
-	fatal("can't fstat message file", EX_OSFILE);
+	fatal("can't fstat message file", EC_OSFILE);
     }
     *basep = 0;
     *lenp = 0;
@@ -243,7 +246,7 @@ int suppresslog;
     if (mailbox->header_fd != -1) {
 	if (fstat(mailbox->header_fd, &sbuf) == -1) {
 	    syslog(LOG_ERR, "IOERROR: fstating %s: %m", fnamebuf);
-	    fatal("can't fstat header file", EX_OSFILE);
+	    fatal("can't fstat header file", EC_OSFILE);
 	}
 	map_refresh(mailbox->header_fd, 1, &mailbox->header_base,
 		    &mailbox->header_len, sbuf.st_size, "header", name);
@@ -521,8 +524,6 @@ mailbox_read_index_header(mailbox)
 struct mailbox *mailbox;
 {
     struct stat sbuf;
-    char buf[INDEX_HEADER_SIZE];
-    int n;
 
     if (mailbox->index_fd == -1) return IMAP_MAILBOX_BADFORMAT;
 
@@ -605,7 +606,7 @@ struct index_record *record;
 	return IMAP_IOERROR;
     }
 
-    buf = mailbox->index_base + offset;
+    buf = (unsigned char*) mailbox->index_base + offset;
 
     record->uid = htonl(*((bit32 *)(buf+OFFSET_UID)));
     record->internaldate = htonl(*((bit32 *)(buf+OFFSET_INTERNALDATE)));
@@ -642,8 +643,7 @@ struct quota *quota;
     }
 
     if (quota->fd == -1) {
-	sprintf(buf, "%s%s%s", config_dir, FNAME_QUOTADIR,
-		quota->root);
+	mailbox_hash_quota(buf, quota->root);
 	quota->fd = open(buf, O_RDWR, 0);
 	if (quota->fd == -1) {
 	    syslog(LOG_ERR, "IOERROR: opening quota file %s: %m", buf);
@@ -819,7 +819,7 @@ struct quota *quota;
 	quota->limit = -1;
 	return 0;
     }
-    sprintf(quota_path, "%s%s%s", config_dir, FNAME_QUOTADIR, quota->root);
+    mailbox_hash_quota(quota_path, quota->root);
     if (quota->fd == -1) {
 	quota->fd = open(quota_path, O_RDWR, 0);
 	if (quota->fd == -1) {
@@ -954,7 +954,7 @@ struct mailbox *mailbox;
 
     if (fstat(mailbox->header_fd, &sbuf) == -1) {
 	syslog(LOG_ERR, "IOERROR: fstating %s: %m", fnamebuf);
-	fatal("can't fstat header file", EX_OSFILE);
+	fatal("can't fstat header file", EC_OSFILE);
     }
     map_refresh(mailbox->header_fd, 1, &mailbox->header_base,
 		&mailbox->header_len, sbuf.st_size, "header", mailbox->name);
@@ -1116,8 +1116,7 @@ struct quota *quota;
 
     if (!quota->root) return 0;
 
-    sprintf(quota_path, "%s%s%s", config_dir, FNAME_QUOTADIR,
-	    quota->root);
+    mailbox_hash_quota(quota_path, quota->root);
     strcpy(new_quota_path, quota_path);
     strcat(new_quota_path, ".NEW");
 
@@ -1172,16 +1171,13 @@ int
 mailbox_delete_quota(quota)
 struct quota *quota;
 {
-    int r;
     char quota_path[MAX_MAILBOX_PATH];
-    FILE *newfile;
 
     assert(quota->lock_count != 0);
 
     if (!quota->root) return 0;
 
-    sprintf(quota_path, "%s%s%s", config_dir, FNAME_QUOTADIR,
-	    quota->root);
+    mailbox_hash_quota(quota_path, quota->root);
 
     unlink(quota_path);
 
@@ -1224,7 +1220,6 @@ void *deciderock;
     unsigned long cache_len;
     unsigned long cache_offset;
     struct stat sbuf;
-    long left;
     char *fnametail;
 
     /* Lock files and open new index/cache files */
@@ -1245,7 +1240,7 @@ void *deciderock;
 
     if (fstat(mailbox->cache_fd, &sbuf) == -1) {
 	syslog(LOG_ERR, "IOERROR: fstating %s: %m", fnamebuf);
-	fatal("can't fstat cache file", EX_OSFILE);
+	fatal("can't fstat cache file", EC_OSFILE);
     }
     cache_len = sbuf.st_size;
     map_refresh(mailbox->cache_fd, 0, &mailbox->cache_base,
@@ -1611,19 +1606,19 @@ mailbox_findquota(name)
 const char *name;
 {
     static char quota_path[MAX_MAILBOX_PATH];
-    char *start, *tail;
+    static char start[MAX_MAILBOX_PATH];
+    char *tail;
     struct stat sbuf;
 
-    strcpy(quota_path, config_dir);
-    strcat(quota_path, FNAME_QUOTADIR);
-    start = quota_path + strlen(quota_path);
     strcpy(start, name);
     lcase(start);
 
+    mailbox_hash_quota(quota_path, start);
     while (stat(quota_path, &sbuf) == -1) {
 	tail = strrchr(start, '.');
 	if (!tail) return 0;
 	*tail = '\0';
+	mailbox_hash_quota(quota_path, start);
     }
     return start;
 }
@@ -2009,6 +2004,7 @@ const char *to;
     int n;
 
     if (link(from, to) == 0) return 0;
+
     destfd = open(to, O_RDWR|O_TRUNC|O_CREAT, 0666);
     if (destfd == -1) {
 	syslog(LOG_ERR, "IOERROR: creating %s: %m", to);
@@ -2045,3 +2041,53 @@ const char *to;
     close(destfd);
     return 0;
 }
+
+void mailbox_hash_mbox(char *buf, 
+		       const char *root,
+		       const char *name)
+{
+    char *idx, *p;
+    char c;
+
+    if (config_hashimapspool) {
+	idx = strchr(name, '.');
+	if (idx == NULL) {
+	    c = *name;
+	} else {
+	    c = *(idx + 1);
+	}
+	c = (char) tolower((int) c);
+	if (!islower(c))
+	    c = 'q';
+	
+	sprintf(buf, "%s/%c/%s", root, c, name);
+    } else {
+	/* standard mailbox placement */
+	sprintf(buf, "%s/%s", root, name);
+    }
+
+    /* change all '.'s to '/' */
+    for (p = buf + strlen(root) + 3; *p; p++) {
+	if (*p == '.') *p = '/';
+    }
+}
+
+/* simple hash so it's easy to find these things in the filesystem;
+   our human time is worth more than efficiency */
+void mailbox_hash_quota(char *buf, const char *qr)
+{
+    char *idx;
+    char c;
+
+    idx = strchr(qr, '.'); /* skip past user. */
+    if (idx == NULL) {
+	c = *qr;
+    } else {
+	c = *(idx + 1);
+    }
+    c = (char) tolower((int) c);
+    if (!islower(c))
+	c = 'q';
+
+    sprintf(buf, "%s%s%c/%s", config_dir, FNAME_QUOTADIR, c, qr);
+}
