@@ -26,16 +26,22 @@
  *
  */
 /*
- * $Id: mboxlist.c,v 1.89 1998/10/29 20:16:32 tjs Exp $
+ * $Id: mboxlist.c,v 1.100 1999/12/30 21:07:26 leg Exp $
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/uio.h>
 #include <fcntl.h>
+#include <ctype.h>
+#include <time.h>
 #include <syslog.h>
 #include <com_err.h>
 
@@ -52,42 +58,10 @@ extern int errno;
 #include "util.h"
 #include "retry.h"
 #include "mailbox.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "imap_err.h"
 #include "xmalloc.h"
 
-/* #define DEBUG /* XXX */
-#ifdef DEBUG
-/* globcheck
-   Called with some of glob_tests's arguments; if it gets called too many
-   times (as indicated by a variable passed in), it bails out, killing
-   imapd by calling fatal.  (glob_test has a bug where it doesn't update
-   minmatch, causing problems.)
- */
-#define NGLOBLIMIT 30000
-void globcheck(globpat, target, tlen, n, line)
-char* globpat;
-char* target;
-int tlen;
-int n;
-char* line;
-{
-    char buf[512];
-    if (!n) {
-	strncpy(buf, target, n);
-	if (tlen > 511) {
-	    tlen = 511;
-	}
-	buf[tlen] = '\0';
-	syslog(LOG_ERR,
-	       "Too many globs in one loop, globbing <%s> against <%s> at line %d",
-	       buf, globpat, line);
-	fatal("too many globs in one loop", EX_SOFTWARE);
-    }
-}
-
-#endif /* DEBUG */
-
 acl_canonproc_t mboxlist_ensureOwnerRights;
 
 static char *listfname, *newlistfname;
@@ -109,6 +83,9 @@ static struct quota *mboxlist_newquota;
 static int mboxlist_changequota();
 static int mboxlist_safe_rename();
 
+static char *mboxlist_hash_usersubs(const char *userid);
+
+
 #define FNAME_MBOXLIST "/mailboxes"
 #define FNAME_USERDIR "/user/"
 #define FNAME_SUBSSUFFIX ".sub"
@@ -122,7 +99,7 @@ static int mboxlist_safe_rename();
 /*
  * Check our configuration for consistency, die if there's a problem
  */
-mboxlist_checkconfig()
+void mboxlist_checkconfig()
 {
     mboxlist_reopen();
 }
@@ -135,14 +112,11 @@ mboxlist_checkconfig()
  * is placed in the char * pointed to by it.  If 'acl' is non-nil, a pointer
  * to the mailbox ACL is placed in the char * pointed to by it.
  */
-mboxlist_lookup(name, pathp, aclp)
-char *name;
-char **pathp;
-char **aclp;
+int mboxlist_lookup(const char* name, char** pathp, char** aclp)
 {
     unsigned long offset, len, partitionlen, acllen;
     char optionbuf[MAX_MAILBOX_NAME+1];
-    char *p, *partition, *acl;
+    char *partition, *acl;
     const char *root;
     static char pathresult[MAX_MAILBOX_PATH];
     static char *aclresult;
@@ -173,10 +147,7 @@ char **aclp;
 	    return IMAP_PARTITION_UNKNOWN;
 	}
 	
-	sprintf(pathresult, "%s/%s", root, name);
-	for (p = pathresult + strlen(root); *p; p++) {
-	    if (*p == '.') *p = '/';
-	}
+	mailbox_hash_mbox(pathresult, root, name);
 
 	*pathp = pathresult;
     }
@@ -197,11 +168,51 @@ char **aclp;
     return 0;
 }
 
+int mboxlist_findstage(const char *name, char *stagedir) 
+{
+    unsigned long offset, len, partitionlen;
+    char optionbuf[MAX_MAILBOX_NAME+1];
+    char *partition;
+    const char *root;
+
+    assert(stagedir != NULL);
+
+    mboxlist_reopen();
+
+    /* Find mailbox */
+    offset = bsearch_mem(name, 1, list_base, list_size, 0, &len);
+    if (!len) {
+	return IMAP_MAILBOX_NONEXISTENT;
+    }
+	
+    /* Parse partition name, construct pathname if requested */
+    mboxlist_parseline(offset, len, NULL, NULL,
+		       &partition, &partitionlen, NULL, NULL);
+
+    if (partitionlen > sizeof(optionbuf)-11) {
+	return IMAP_PARTITION_UNKNOWN;
+    }
+    strcpy(optionbuf, "partition-");
+    memcpy(optionbuf + 10, partition, partitionlen);
+    optionbuf[10+partitionlen] = '\0';
+    
+    root = config_getstring(optionbuf, (char *)0);
+    if (!root) {
+	return IMAP_PARTITION_UNKNOWN;
+    }
+	
+    strcpy(stagedir, root);
+    strcat(stagedir, "/");
+    sprintf(stagedir, "%s/stage./", root);
+    
+    return 0;
+}
+
 /*
  * Check/set up for mailbox creation
  */
-mboxlist_createmailboxcheck(name, format, partition, isadmin, userid,
-			    auth_state, newacl, newpartition)
+int mboxlist_createmailboxcheck(name, format, partition, isadmin, userid,
+				auth_state, newacl, newpartition)
 char *name;
 int format;
 char *partition;
@@ -342,7 +353,7 @@ char **newpartition;
 	    partition = (char *)config_defpartition;
 	    if (strlen(partition) > MAX_PARTITION_LEN) {
 		/* Configuration error */
-		fatal("name of default partition is too long", EX_CONFIG);
+		fatal("name of default partition is too long", EC_CONFIG);
 	    }
 	}
 	partition = xstrdup(partition);
@@ -359,7 +370,8 @@ char **newpartition;
 /*
  * Create a mailbox
  */
-mboxlist_createmailbox(name, format, partition, isadmin, userid, auth_state)
+int mboxlist_createmailbox(name, format, partition, isadmin, 
+			   userid, auth_state)
 char *name;
 int format;
 char *partition;
@@ -368,7 +380,6 @@ char *userid;
 struct auth_state *auth_state;
 {
     int r;
-    char *p;
     unsigned long offset, len;
     char *acl;
     char buf2[MAX_MAILBOX_PATH];
@@ -451,10 +462,7 @@ struct auth_state *auth_state;
     }
 
     /* Create new mailbox and move new mailbox list file into place */
-    sprintf(buf2, "%s/%s", root, name);
-    for (p = buf2 + strlen(root); *p; p++) {
-	if (*p == '.') *p = '/';
-    }
+    mailbox_hash_mbox(buf2, root, name);
     r = mailbox_create(name, buf2, acl, format, &newmailbox);
     free(acl);
 
@@ -486,7 +494,7 @@ struct auth_state *auth_state;
  * performed by an admin.  The operation removes the user "FOO"'s 
  * subscriptions and all sub-mailboxes of user.FOO
  */
-mboxlist_deletemailbox(name, isadmin, userid, auth_state, checkacl)
+int mboxlist_deletemailbox(name, isadmin, userid, auth_state, checkacl)
 char *name;
 int isadmin;
 char *userid;
@@ -533,12 +541,8 @@ int checkacl;
 	{
 	    char *fname;
 
-	    fname = xmalloc(strlen(config_dir)+sizeof(FNAME_USERDIR)+
-				strlen(name)+sizeof(FNAME_SUBSSUFFIX));
-	    strcpy(fname, config_dir);
-	    strcat(fname, FNAME_USERDIR);
-	    strcat(fname, name+5);
-	    strcat(fname, FNAME_SUBSSUFFIX);
+	    fname = mboxlist_hash_usersubs(name + 5);
+
 	    (void) unlink(fname);
 	    free(fname);
 	}
@@ -683,7 +687,6 @@ struct auth_state *auth_state;
     long access;
     int isusermbox = 0;
     char *oldpath;
-    char *p;
     unsigned long oldoffset, oldlen;
     unsigned long newoffset, newlen;
     bit32 olduidvalidity, newuidvalidity;
@@ -852,10 +855,8 @@ struct auth_state *auth_state;
     }
 
     /* Rename the mailbox and move new mailbox list file into place */
-    sprintf(buf2, "%s/%s", root, newname);
-    for (p = buf2 + strlen(root); *p; p++) {
-	if (*p == '.') *p = '/';
-    }
+    mailbox_hash_mbox(buf2, root, newname);
+
     r = mailbox_rename(oldname, newname, buf2, isusermbox,
 		       &olduidvalidity, &newuidvalidity);
     if (r) {
@@ -1072,9 +1073,6 @@ void* rock;
     int rights;
     int r;
     char *inboxcase;
-#ifdef DEBUG
-    int nglobtests = NGLOBLIMIT;
-#endif /* DEBUG */
 
     mboxlist_reopen();
     list_doingfind++;
@@ -1163,9 +1161,6 @@ void* rock;
 	    if (strncmp(list_base + offset,
 			    usermboxname, usermboxnamelen) != 0) break;
 	    minmatch = 0;
-#ifdef DEBUG
-	    nglobtests = NGLOBLIMIT;
-#endif /* DEBUG */
 	    while (minmatch >= 0) {
 		memcpy(namebuf, name, namelen);
 		namebuf[namelen] = '\0';
@@ -1208,15 +1203,12 @@ void* rock;
 	mboxlist_parseline(offset, len, &name, &namelen,
 			   (char **)0, (unsigned long *)0, &acl, &acllen);
 		
-	if (strncmp(list_base + offset, pattern, prefixlen)) break;
+	if (strncmp(list_base + offset, pattern, prefixlen)) {
+	    /* doesn't match the constant prefix */
+	    break;
+	}
 	minmatch = 0;
-#ifdef DEBUG
-	nglobtests = NGLOBLIMIT;
-#endif /* DEBUG */
 	while (minmatch >= 0) {
-#ifdef DEBUG
-	    globcheck(pattern, name, namelen, nglobtests--, __LINE__);
-#endif /* DEBUG */
 	    matchlen = glob_test(g, name, namelen, &minmatch);
 
 	    if (matchlen == -1 ||
@@ -1224,6 +1216,10 @@ void* rock;
 		 strncmp(name, usermboxname, usermboxnamelen) == 0 &&
 		 (namelen == usermboxnamelen ||
 		  name[usermboxnamelen] == '.'))) {
+		/* we didn't match
+		        OR
+		   this is one of my personal mailboxes & 
+		   we already listed it */
 		break;
 	    }
 
@@ -1262,6 +1258,7 @@ void* rock;
 		}
 	    }
 	}
+	/* go to the next record in the mailboxes file */
 	offset += len;
     }
 	
@@ -1301,9 +1298,6 @@ int (*proc)();
     long matchlen, minmatch;
     char *acl;
     char *inboxcase;
-#ifdef DEBUG
-    int nglobtests = NGLOBLIMIT;
-#endif /* DEBUG */
 
     if (r = mboxlist_opensubs(userid, 0, &subsfd, &subs_base, &subs_size,
 			      &subsfname, (char **) 0)) {
@@ -1392,7 +1386,7 @@ int (*proc)();
 	    if (!p || !endname || endname - name > MAX_MAILBOX_NAME) {
 		syslog(LOG_ERR, "IOERROR: corrupted subscription file %s",
 		       subsfname);
-		fatal("corrupted subscription file", EX_OSFILE);
+		fatal("corrupted subscription file", EC_OSFILE);
 	    }
 
 	    len = p - name + 1;
@@ -1413,10 +1407,6 @@ int (*proc)();
 		    namematchbuf[inboxoffset+4] = inboxcase[4];
 		}
 
-#ifdef DEBUG
-		globcheck(pattern, name, namelen, nglobtests--, __LINE__);
-#endif /* DEBUG */
-		
 		matchlen = glob_test(g, namematchbuf+inboxoffset,
 				     namelen-inboxoffset, &minmatch);
 		if (matchlen == -1) break;
@@ -1454,7 +1444,7 @@ int (*proc)();
 	if (!p || !endname || endname - name > MAX_MAILBOX_NAME) {
 	    syslog(LOG_ERR, "IOERROR: corrupted subscription file %s",
 		   subsfname);
-	    fatal("corrupted subscription file", EX_OSFILE);
+	    fatal("corrupted subscription file", EC_OSFILE);
 	}
 
 	len = p - name + 1;
@@ -1462,13 +1452,7 @@ int (*proc)();
 
 	if (strncmp(name, pattern, prefixlen)) break;
 	minmatch = 0;
-#ifdef DEBUG
-	nglobtests = NGLOBLIMIT;
-#endif
 	while (minmatch >= 0) {
-#ifdef DEBUG
-	    globcheck(pattern, name, namelen, nglobtests--, __LINE__);
-#endif
 	    matchlen = glob_test(g, name, namelen, &minmatch);
 	    if (matchlen == -1 ||
 		(userid && namelen >= usermboxnamelen &&
@@ -1621,10 +1605,8 @@ int newquota;
     
     quota = zeroquota;
 
-    strcpy(quota_path, config_dir);
-    strcat(quota_path, FNAME_QUOTADIR);
-    quota.root = quota_path + strlen(quota_path);
-    strcpy(quota.root, root);
+    quota.root = (char *) root;
+    mailbox_hash_quota(quota_path, root);
 
     if ((quota.fd = open(quota_path, O_RDWR, 0)) != -1) {
 	/* Just lock and change it */
@@ -1876,7 +1858,7 @@ mboxlist_unlock()
 /*
  * Retrieve internal information, for reconstructing mailboxes file
  */
-mboxlist_getinternalstuff(listfnamep, newlistfnamep, basep, sizep)
+void mboxlist_getinternalstuff(listfnamep, newlistfnamep, basep, sizep)
 const char **listfnamep;
 const char **newlistfnamep;
 const char **basep;
@@ -1934,12 +1916,8 @@ const char **newfname;
     }
 
     /* Build subscription list filename */
-    subsfname = xmalloc(strlen(config_dir)+sizeof(FNAME_USERDIR)+
-			strlen(userid)+sizeof(FNAME_SUBSSUFFIX));
-    strcpy(subsfname, config_dir);
-    strcat(subsfname, FNAME_USERDIR);
-    strcat(subsfname, userid);
-    strcat(subsfname, FNAME_SUBSSUFFIX);
+    subsfname = mboxlist_hash_usersubs(userid);
+
     newsubsfname = xmalloc(strlen(subsfname)+5);
     strcpy(newsubsfname, subsfname);
     strcat(newsubsfname, ".NEW");
@@ -1961,7 +1939,7 @@ const char **newfname;
     else {
 	if (fstat(subsfd, &sbuf) == -1) {
 	    syslog(LOG_ERR, "IOERROR: fstat on %s: %m", subsfname);
-	    fatal("can't fstat subscription list", EX_OSFILE);
+	    fatal("can't fstat subscription list", EC_OSFILE);
 	}
     }
 
@@ -1994,7 +1972,7 @@ unsigned long size;
  * use when updating the mailbox list.
  */
 static void
-mboxlist_reopen()
+mboxlist_reopen(void)
 {
     struct stat sbuf;
 
@@ -2013,7 +1991,9 @@ mboxlist_reopen()
     if (listfd != -1) {
 	if (stat(listfname, &sbuf) == -1) {
 	    syslog(LOG_ERR, "IOERROR: stat on %s: %m", listfname);
-	    fatal("can't stat mailboxes file", EX_OSFILE);
+	    /* Exiting TEMPFAIL because Sendmail thinks this
+	       EC_OSFILE == permenant failure. */
+	    fatal("can't stat mailboxes file", EC_TEMPFAIL);
 	}
 	if (sbuf.st_ino == list_ino) return;
 	close(listfd);
@@ -2023,12 +2003,14 @@ mboxlist_reopen()
     listfd = open(listfname, O_RDWR, 0666);
     if (listfd == -1) {
 	syslog(LOG_ERR, "IOERROR: opening %s: %m", listfname);
-	fatal("can't read mailboxes file", EX_OSFILE);
+	    /* Exiting TEMPFAIL because Sendmail thinks this
+	       EC_OSFILE == permenant failure. */
+	fatal("can't read mailboxes file", EC_TEMPFAIL);
     }
 	
     if (fstat(listfd, &sbuf) == -1) {
 	syslog(LOG_ERR, "IOERROR: fstat on %s: %m", listfname);
-	fatal("can't fstat mailboxes file", EX_OSFILE);
+	fatal("can't fstat mailboxes file", EC_OSFILE);
     }
     list_ino = sbuf.st_ino;
 
@@ -2052,7 +2034,7 @@ char *error;
     syslog(LOG_ERR, "IOERROR: corrupted mailboxes file, line %d: %s",
 	   lineno, error);
     sprintf(buf, "corrupted mailboxes file, line %d: %s", lineno, error);
-    fatal(buf, EX_OSFILE);
+    fatal(buf, EC_OSFILE);
 }
     
 
@@ -2198,15 +2180,15 @@ int maycreate;
    files in memory, increasingly inaccurate, that would simply be thrown
    out at some future time.  */
 void
-mboxlist_close()
+mboxlist_close(void)
 {
-  if (list_size && list_base && *list_base) {
-    map_free(&list_base, &list_size);
-  }
-  if (listfd != -1) {
-    close(listfd);
-    listfd = -1;
-  }
+    if (list_size && list_base && *list_base) {
+	map_free(&list_base, &list_size);
+    }
+    if (listfd != -1) {
+	close(listfd);
+	listfd = -1;
+    }
 }
 
 /* Safe rename, for file being renamed to a file that might get locked
@@ -2235,3 +2217,19 @@ int fd;
     lock_unlock(fd);
     return 0;
 }
+
+static char *mboxlist_hash_usersubs(const char *userid)
+{
+    char *fname = xmalloc(strlen(config_dir) + sizeof(FNAME_USERDIR) +
+			  strlen(userid) + sizeof(FNAME_SUBSSUFFIX) + 10);
+    char c;
+
+    c = (char) tolower((int) *userid);
+    if (!islower(c)) {
+	c = 'q';
+    }
+    sprintf(fname, "%s%s%c/%s%s", config_dir, FNAME_USERDIR, c, userid,
+	    FNAME_SUBSSUFFIX);
+
+    return fname;
+}
