@@ -27,7 +27,7 @@
  */
 
 /*
- * $Id: message.c,v 1.59 1998/10/29 20:16:31 tjs Exp $
+ * $Id: message.c,v 1.70 1999/10/27 21:04:18 leg Exp $
  */
 
 #ifdef HAVE_UNISTD_H
@@ -42,8 +42,9 @@
 #include <sys/uio.h>
 #include <sys/stat.h>
 #include <netinet/in.h>
+#include <stdlib.h>
 
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "imap_err.h"
 #include "prot.h"
 #include "map.h"
@@ -53,6 +54,7 @@
 #include "charset.h"
 #include "util.h"
 #include "xmalloc.h"
+#include "config.h"
 
 /* Message being parsed */
 struct msg {
@@ -215,7 +217,7 @@ unsigned size;
     while (size) {
 	n = prot_read(from, buf, size > 4096 ? 4096 : size);
 	if (!n) {
-	    syslog(LOG_ERR, "IOERROR: reading message: %m");
+	    syslog(LOG_ERR, "IOERROR: reading message: unexpected end of file");
 	    return IMAP_IOERROR;
 	}
 
@@ -310,7 +312,7 @@ struct index_record *message_index;
     if (fstat(fileno(infile), &sbuf) == -1) {
 	syslog(LOG_ERR, "IOERROR: fstat on new message in %s: %m",
 	       mailbox->name);
-	fatal("can't fstat message file", EX_OSFILE);
+	fatal("can't fstat message file", EC_OSFILE);
     }
     map_refresh(fileno(infile), 1, &msg_base, &msg_len, sbuf.st_size,
 		"new message", mailbox->name);
@@ -364,7 +366,7 @@ struct index_record *message_index;
 /*
  * Parse a body-part
  */
-static 
+static int
 message_parse_body(msg, format, body, defaultContentType, boundaries)
 struct msg *msg;
 int format;
@@ -590,6 +592,10 @@ struct boundary *boundaries;
 		if (!strncasecmp(next+2, "rom:", 4)) {
 		    message_parse_address(next+6, &body->from);
 		}
+		else if (body->cacheheaders.start &&
+			 !strncasecmp(next+2, "ollowup-to:", 11)) {
+		    message_parse_header(next+1, &body->cacheheaders);
+		}
 		break;
 
 	    case 'i':
@@ -606,6 +612,14 @@ struct boundary *boundaries;
 		}
 		break;
 
+	    case 'n':
+	    case 'N':
+		if (body->cacheheaders.start &&
+		    !strncasecmp(next+2, "ewsgroups:", 10)) {
+		    message_parse_header(next+1, &body->cacheheaders);
+		}
+		break;
+		
 	    case 'p':
 	    case 'P':
 		if (body->cacheheaders.start &&
@@ -623,6 +637,11 @@ struct boundary *boundaries;
 			 !strncasecmp(next+2, "eferences:", 10)) {
 		    message_parse_header(next+1, &body->cacheheaders);
 		}
+		else if (body->cacheheaders.start &&
+			 !strncasecmp(next+2, "esent-from:", 11)) {
+		    message_parse_header(next+1, &body->cacheheaders);
+		}
+
 		break;
 
 	    case 's':
@@ -900,7 +919,6 @@ struct body *body;
 
     /* Skip whitespace after type */
     message_parse_rfc822space(&hdr);
-    if (!hdr) return;
 
     /* Ignore header if not at end of header or parameter delimiter */
     if (hdr && *hdr != ';') return;
@@ -969,7 +987,8 @@ struct param **paramp;
 		    if (!*hdr) return;
 		}
 		if (*hdr == '\r') {
-		    return;
+		    if (hdr[1] == '\n' && (hdr[2] == ' ' || hdr[2] == '\t')) hdr += 2;
+ 		    else return;
 		}
 		hdr++;
 	    }
@@ -1003,6 +1022,7 @@ struct param **paramp;
 	    value++;
 	    while (*value != '\"') {
 		if (*value == '\\') value++;
+		else if (*value == '\r') value += 2;
 		*p++ = *value++;
 	    }
 	    *p = '\0';
@@ -1420,11 +1440,17 @@ struct boundary *boundaries;
     message_parse_content(msg, format, &preamble, boundaries);
 
     /* Parse the component body-parts */
-    while (boundaries->count == depth && msg->offset < msg->len) {
+    while (boundaries->count == depth) {
 	body->subpart = (struct body *)xrealloc((char *)body->subpart,
 				 (body->numparts+1)*sizeof(struct body));
 	message_parse_body(msg, format, &body->subpart[body->numparts++],
 			   defaultContentType, boundaries);
+	if (msg->offset == msg->len &&
+	    body->subpart[body->numparts-1].boundary_size == 0) {
+	    /* hit the end of the message, therefore end all pending
+	       multiparts */
+	    boundaries->count = 0;
+	}
     }
 
     if (boundaries->count == depth-1) {
@@ -1591,6 +1617,7 @@ struct body *body;
     struct body toplevel;
     int n;
     struct iovec iov[15];
+    char* t;
 
     toplevel.type = "MESSAGE";
     toplevel.subtype = "RFC822";
@@ -1621,8 +1648,9 @@ struct body *body;
     message_write_searchaddr(&bcc, body->bcc);
 
     message_ibuf_init(&subject);
-    message_write_nstring(&subject, charset_decode1522(body->subject));
-
+    t = charset_decode1522(body->subject, NULL, 0);
+    message_write_nstring(&subject, t);
+    free(t);
 
     message_ibuf_iov(&iov[0], &envelope);
     message_ibuf_iov(&iov[1], &bodystructure);
@@ -2166,6 +2194,7 @@ struct ibuf *ibuf;
 struct address *addrlist;
 {
     int prevaddr = 0;
+    char* tmp;
 
     while (addrlist) {
 
@@ -2174,8 +2203,10 @@ struct address *addrlist;
 	    if (addrlist->mailbox) {
 		if (prevaddr) PUTIBUF(ibuf, ',');
 		
-		message_write_text(ibuf,
-				   charset_decode1522(addrlist->mailbox));
+		tmp = charset_decode1522(addrlist->mailbox, NULL, 0);
+		message_write_text(ibuf, tmp);
+		free(tmp);
+		tmp = NULL;
 		PUTIBUF(ibuf, ':');
 	
 		/* Suppress a trailing comma */
@@ -2190,7 +2221,9 @@ struct address *addrlist;
 	    if (prevaddr) PUTIBUF(ibuf, ',');
 
 	    if (addrlist->name) {
-		message_write_text(ibuf, charset_decode1522(addrlist->name));
+		tmp = charset_decode1522(addrlist->name, NULL, 0);
+		message_write_text(ibuf, tmp);
+		free(tmp); tmp = NULL;
 		PUTIBUF(ibuf, ' ');
 	    }
 
@@ -2262,7 +2295,6 @@ struct ibuf *ibuf;
 {
     char *s;
     int len;
-    int pad;
 
     /* Drop padding on end */
     message_ibuf_ensure(ibuf, 3);
