@@ -1,5 +1,5 @@
 /* notify_zephyr.c -- Module to notify of new mail via zephyr
- $Id: notify_zephyr.c,v 1.13 1998/05/15 21:49:33 neplokh Exp $
+ $Id: notify_zephyr.c,v 1.16 1999/10/02 00:43:06 leg Exp $
  
  # Copyright 1998 Carnegie Mellon University
  # 
@@ -31,14 +31,14 @@
 #include <string.h>
 #include <netdb.h>
 #include <sys/types.h>
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_LIBKRB
 #include <netinet/in.h>
 #include <krb.h>
 #endif
 #include <zephyr/zephyr.h>
 #include <syslog.h>
 
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "xmalloc.h"
 
 #ifndef MAIL_CLASS
@@ -49,21 +49,19 @@ extern int errno;
 
 /* This code is mostly stolen from zpopnotify, from the Zephyr dist. */
 
-notify_wantheader()
+int notify_wantheader()
 {
     return 1;
 }
 
-notify(user, mailbox, header)
+void notify(user, mailbox, header)
 char *user;
 char *mailbox;
 char *header;
 {
     ZNotice_t notice;
-    struct hostent *hent;
     int retval;
-    register int i;
-    char *whoami,myhost[256],mysender[BUFSIZ];
+    char myhost[256],mysender[BUFSIZ];
     char *msgbody;
     char *lines[2];
     char *mykrbhost = 0;
@@ -79,7 +77,7 @@ char *header;
     }
     myhost[sizeof(myhost)-1] = '\0';
   
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_LIBKRB
     mykrbhost = krb_get_phost(myhost);
 #endif
   
