@@ -26,9 +26,14 @@
  */
 
 /*
- * $Id: pop3d.c,v 1.40 1998/12/02 18:54:46 tjs Exp $
+ * $Id: pop3d.c,v 1.54 1999/12/29 19:20:55 leg Exp $
  */
 
+#ifndef __GNUC__
+/* can't use attributes... */
+#define __attribute__(foo)
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
@@ -42,13 +47,20 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
+#include <sasl.h>
+
 #include "acl.h"
 #include "util.h"
 #include "auth.h"
-#include "sasl.h"
 #include "config.h"
+
+/* openSSL has it's own DES function which conflict in names with those used by krb.h */
+#ifdef HAVE_SSL
+#undef HAVE_SSL
+#endif /* HAVE_SSL */
+
 #include "prot.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "version.h"
@@ -62,7 +74,7 @@ extern int errno;
 
 extern char *login_capabilities();
 
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_KRB
 #include <krb.h>
 
 /* MIT's kpop authentication kludge */
@@ -71,6 +83,8 @@ char klrealm[REALM_SZ];
 AUTH_DAT kdata;
 #endif
 
+sasl_conn_t *popd_saslconn; /* the sasl connection context */
+
 char *popd_userid = 0;
 struct mailbox *popd_mailbox = 0;
 struct sockaddr_in popd_localaddr, popd_remoteaddr;
@@ -95,6 +109,68 @@ static void cmd_capa();
 static void cmd_pass();
 static void cmd_user();
 
+/* This creates a structure that defines the allowable
+ *   security properties 
+ */
+static sasl_security_properties_t *make_secprops(int min,int max)
+{
+  sasl_security_properties_t *ret=
+    (sasl_security_properties_t *) xmalloc(sizeof(sasl_security_properties_t));
+
+  ret->maxbufsize=4000;
+  ret->min_ssf=min;     /* minimum allowable security strength */
+  ret->max_ssf=max;     /* maximum allowable security strength */
+
+  ret->security_flags = 0;
+  if (!config_getswitch("allowplaintext", 1)) {
+      ret->security_flags |= SASL_SEC_NOPLAINTEXT;
+  }
+  if (!config_getswitch("allowanonymouslogin", 0)) {
+      ret->security_flags |= SASL_SEC_NOANONYMOUS;
+  }
+  ret->property_names=NULL;
+  ret->property_values=NULL;
+
+  return ret;
+}
+
+/* this is a wrapper to call the cyrus configuration from SASL */
+static int mysasl_config(void *context __attribute__((unused)), 
+			 const char *plugin_name,
+			 const char *option,
+			 const char **result,
+			 unsigned *len)
+{
+    char opt[1024];
+
+    if (strcmp(option, "srvtab")) { /* we don't transform srvtab! */
+	int sl = 5 + (plugin_name ? strlen(plugin_name) + 1 : 0);
+
+	strncpy(opt, "sasl_", 1024);
+	if (plugin_name) {
+	    strncat(opt, plugin_name, 1019);
+	    strncat(opt, "_", 1024 - sl);
+	}
+ 	strncat(opt, option, 1024 - sl - 1);
+	opt[1023] = '\0';
+    } else {
+	strncpy(opt, option, 1024);
+    }
+
+    *result = config_getstring(opt, NULL);
+    if (*result != NULL) {
+	if (len) { *len = strlen(*result); }
+	return SASL_OK;
+    }
+   
+    return SASL_FAIL;
+}
+
+static struct sasl_callback mysasl_cb[] = {
+    { SASL_CB_GETOPT, &mysasl_config, NULL },
+    { SASL_CB_LIST_END, NULL, NULL }
+};
+
 main(argc, argv, envp)
 int argc;
 char **argv;
@@ -106,16 +182,17 @@ char **envp;
     struct hostent *hp;
     struct sockaddr_in sa;
     int timeout;
+    sasl_security_properties_t *secprops=NULL;
 
     popd_in = prot_new(0, 0);
     popd_out = prot_new(1, 1);
 
-    if (geteuid() == 0) fatal("must run as the Cyrus user", EX_USAGE);
+    if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
     opterr = 0;
     while ((opt = getopt(argc, argv, "k")) != EOF) {
 	switch(opt) {
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_KRB
 	case 'k':
 	    kflag++;
 	    break;
@@ -158,6 +235,23 @@ char **envp;
 	}
     }
 
+    /* Make a SASL connection and setup some properties for it */
+
+    if (sasl_server_init(mysasl_cb, "Cyrus") != SASL_OK)
+	fatal("SASL failed initializing: sasl_server_init()",EC_TEMPFAIL); 
+
+    /* other params should be filled in */
+    if (sasl_server_new("pop", hostname, NULL, 
+			NULL, 2000, &popd_saslconn)!=SASL_OK)
+	fatal("SASL failed initializing: sasl_server_new()",EC_TEMPFAIL); 
+
+    /* will always return something valid */
+    secprops=make_secprops(0,2000);        
+    sasl_setprop(popd_saslconn, SASL_SEC_PROPS, secprops);
+    
+    sasl_setprop(popd_saslconn,   SASL_IP_REMOTE, &popd_remoteaddr);  
+    sasl_setprop(popd_saslconn,   SASL_IP_LOCAL, &popd_localaddr);  
+
     proc_register("pop3d", popd_clienthost, (char *)0, (char *)0);
 
     /* Set inactivity timer */
@@ -166,7 +260,7 @@ char **envp;
     prot_settimeout(popd_in, timeout*60);
     prot_setflushonread(popd_in, popd_out);
 
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_KRB
     if (kflag) kpop();
 #endif
 
@@ -179,21 +273,21 @@ char **envp;
 usage()
 {
     prot_printf(popd_out, "-ERR usage: pop3d%s\r\n",
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_KRB
 		" [-k]"
 #else
 		""
 #endif
 		);
     prot_flush(popd_out);
-    exit(EX_USAGE);
+    exit(EC_USAGE);
 }
 
 /*
  * Cleanly shut down and exit
  */
-shut_down(code)
-int code;
+void shut_down(int code) __attribute__((noreturn));
+void shut_down(int code)
 {
     proc_cleanup();
     if (popd_mailbox) {
@@ -203,9 +297,7 @@ int code;
     exit(code);
 }
 
-fatal(s, code)
-char *s;
-int code;
+void fatal(const char* s, int code)
 {
     static int recurse_code = 0;
 
@@ -220,7 +312,7 @@ int code;
     shut_down(code);
 }
 
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_KRB
 /*
  * MIT's kludge of a kpop protocol
  * Client does a krb_sendauth() first thing
@@ -235,7 +327,7 @@ kpop()
     int r;
 
     if (!popd_haveaddr) {
-	fatal("Cannot get client's IP address");
+	fatal("Cannot get client's IP address", EC_OSERR);
     }
 
     srvtab = config_getstring("srvtab", "");
@@ -292,7 +384,12 @@ cmdloop()
 	for (p = inputbuf; *p && !isspace(*p); p++);
 	if (*p) {
 	    *p++ = '\0';
-	    for (arg=p; *arg && isspace(*arg); arg++);
+	    arg = p;
+	    if (strcasecmp(inputbuf, "pass") != 0) {
+		while (*arg && isspace(*arg)) {
+		    arg++;
+		}
+	    }
 	    if (!*arg) {
 		prot_printf(popd_out, "-ERR Syntax error\r\n");
 		continue;
@@ -557,7 +654,7 @@ char *pass;
 	return;
     }
 
-#ifdef HAVE_SASL_KRB
+#ifdef HAVE_KRB
     if (kflag) {
 	if (strcmp(popd_userid, kdata.pname) != 0 ||
 	    kdata.pinst[0] ||
@@ -592,15 +689,21 @@ char *pass;
 	    return;
 	}
     }
-    else if (login_plaintext(popd_userid, pass, &reply) != 0) {
+    else if ((sasl_checkpass(popd_saslconn,
+			     popd_userid,
+			     strlen(popd_userid),
+			     pass,
+			     strlen(pass),
+			     (const char **) &reply))!=SASL_OK) { 
 	if (reply) {
 	    syslog(LOG_NOTICE, "badlogin: %s plaintext %s %s",
 		   popd_clienthost, popd_userid, reply);
 	}
-	free(popd_userid);
-	popd_userid = 0;
 	sleep(3);
 	prot_printf(popd_out, "-ERR Invalid login\r\n");
+	free(popd_userid);
+	popd_userid = 0;
+
 	return;
     }
     else {
@@ -622,27 +725,24 @@ cmd_capa()
     int minpoll = config_getint("popminpoll", 0) * 60;
     int expire = config_getint("popexpiretime", -1);
     const char *capabilities, *next_capabilities;
-
+    char *sasllist;
+    
     prot_printf(popd_out, "+OK List of capabilities follows\r\n");
 
+#if 0
     /* SASL special case: print SASL, then a list of supported capabilities
        (parsed from the IMAP-style login_capabilities function, then a CRLF */
-    prot_printf(popd_out, "SASL"); /* no \r\n */
-    next_capabilities = login_capabilities();
-    while (next_capabilities[0]) {
-	capabilities = next_capabilities;
-	next_capabilities = strchr(capabilities+1, ' ');
-	if (!next_capabilities) {
-	    next_capabilities = capabilities + strlen(capabilities);
-	}
-	if (!strncmp(capabilities, " AUTH=", 6)) {
-	    capabilities += 6;
-	    prot_putc(' ', popd_out);
-	    prot_write(popd_out, capabilities,
-		       next_capabilities - capabilities);
-	}
+    if (sasl_listmech(popd_saslconn,
+			 "", /* should be id string */
+			 ""," ","",
+			 &sasllist,
+			 NULL,NULL)==SASL_OK)
+    {
+      prot_printf(popd_out, "SASL"); /* no \r\n */
+      prot_printf(popd_out," %s",sasllist);
+      prot_printf(popd_out, "\r\n");    
     }
-    prot_printf(popd_out, "\r\n");
+#endif
 
     if (expire < 0) {
 	prot_printf(popd_out, "EXPIRE NEVER\r\n");
@@ -653,7 +753,7 @@ cmd_capa()
     prot_printf(popd_out, "LOGIN-DELAY %d\r\n", minpoll);
     prot_printf(popd_out, "TOP\r\n");
     prot_printf(popd_out, "UIDL\r\n");
-    prot_printf(popd_out, "PIPELINE\r\n");
+    prot_printf(popd_out, "PIPELINING\r\n");
     prot_printf(popd_out, "USER\r\n");
     
     prot_printf(popd_out,
@@ -664,137 +764,149 @@ cmd_capa()
     prot_flush(popd_out);
 }
 
+
+
+
 void
 cmd_auth(authtype)
 char *authtype;
 {
-    char *initial_response;
-    char *canon_user;
-    int r;
-    struct sasl_server *mech;
-    int (*authproc)();
-    int outputlen;
-    char *output;
-    int inputlen;
-    char *input;
-    void *state;
-    const char *reply = 0;
-    int protlevel;
-    char *user;
-    sasl_encodefunc_t *encodefunc;
-    sasl_decodefunc_t *decodefunc;
-    int maxplain;
-    char *val;
+    int sasl_result;
+    char * clientin;
+    int clientinlen=0;
+    
+    char *serverout;
+    unsigned int serveroutlen;
+    const char *errstr;
+    
+    const char *errorstring = NULL;
 
+    char buf[MAX_MAILBOX_PATH];
+    char *p;
+    FILE *logfile;
+
+    int *ssfp;
+    char *ssfmsg=NULL;
+
+   
+    /* if client didn't specify an auth mechanism we give them the list */
     if (!authtype) {
-	const char *capabilities, *next_capabilities;
+      char *sasllist;
+      unsigned int mechnum;
 
-	prot_printf(popd_out, "+OK List of supported mechanisms follows\r\n");
-	next_capabilities = login_capabilities();
-	while (next_capabilities[0]) {
-	    capabilities = next_capabilities;
-	    next_capabilities = strchr(capabilities+1, ' ');
-	    if (!next_capabilities) {
-		next_capabilities = capabilities + strlen(capabilities);
-	    }
-	    if (!strncmp(capabilities, " AUTH=", 6)) {
-		capabilities += 6;
-		prot_write(popd_out, capabilities,
-			   next_capabilities - capabilities);
-		prot_printf(popd_out, "\r\n");
-	    }
-	}
-	prot_printf(popd_out, ".\r\n");
+      prot_printf(popd_out, "+OK List of supported mechanisms follows\r\n");
+      
+      /* SASL special case: print SASL, then a list of supported capabilities
+       * (parsed from the IMAP-style login_capabilities function, 
+       * then a CRLF */
+      if (sasl_listmech(popd_saslconn,
+			"", /* should be id string */
+			"","\r\n","\r\n",
+			&sasllist,
+			NULL,&mechnum) == SASL_OK) {
+	  if (mechnum>0) {
+	      prot_printf(popd_out,"%s",sasllist);
+	  }
+      }
+      
+      prot_printf(popd_out, ".\r\n");
+      
+      return;
+    }
+
+    /* server did specify a command, so let's try to authenticate */
+
+    sasl_result = sasl_server_start(popd_saslconn, authtype,
+				    NULL, 0,
+				    &serverout, &serveroutlen,
+				    &errstr);    
+
+    /* sasl_server_start will return SASL_OK or SASL_CONTINUE on success */
+
+    while (sasl_result == SASL_CONTINUE)
+    {
+
+      /* print the message to the user */
+      printauthready(serveroutlen, (unsigned char *)serverout);
+      free(serverout);
+
+      /* get string from user */
+      clientinlen = readbase64string(&clientin);
+      if (clientinlen == -1) {
+	prot_printf(popd_out, "-ERR Invalid base64 string\r\n");
 	return;
+      }
+
+      sasl_result = sasl_server_step(popd_saslconn,
+				     clientin,
+				     clientinlen,
+				     &serverout, &serveroutlen,
+				     &errstr);
     }
 
-    if ((initial_response = strchr(authtype, ' '))) {
-	*initial_response++ = '\0';
-    }
-    lcase(authtype);
-    
-    r = login_authenticate(authtype, &mech, &authproc, &reply);
-    if (!r) {
-	r = mech->start(mech->rock, "pop", authproc,
-			SASL_PROT_ANY, PROT_BUFSIZE,
-			popd_haveaddr ? (struct sockaddr *)&popd_localaddr : 0,
-			popd_haveaddr ? (struct sockaddr *)&popd_remoteaddr : 0,
-			&outputlen, &output, &state, &reply);
-    }
-    if (r && r != SASL_DONE) {
-	if (reply) {
+
+    /* failed authentication */
+    if (sasl_result != SASL_OK)
+    {
+	syslog(LOG_NOTICE, "badlogin: %s %s %i",
+	       popd_clienthost, authtype, sasl_result);
+	
+	if (clientin) {
 	    syslog(LOG_NOTICE, "badlogin: %s %s %s",
-		   popd_clienthost, authtype, reply);
+		   popd_clienthost, authtype, clientin);
+	} else {
+	    syslog(LOG_NOTICE, "badlogin: %s %s",
+		   popd_clienthost, authtype);
 	}
-	prot_printf(popd_out, "-ERR Invalid login\r\n");
+	
+	sleep(3);      
+	
+	/* convert the sasl error code to a string */
+	errorstring = sasl_errstring(sasl_result, NULL, NULL);
+      
+	if (errorstring)
+	    prot_printf(popd_out, "-ERR %s\r\n",errorstring);
+	else
+	    prot_printf(popd_out, "-ERR Error Authenticating\r\n");
+	
 	return;
     }
 
-    if (initial_response) {
-	if (outputlen != 0) {
-	    prot_printf(popd_out,
-		   "-ERR Cannot give initial response to this mechanism\r\n");
-	    mech->free_state(state);
-	    return;
-	}
+    /* successful authentication */
 
-	inputlen = parsebase64string(&input, initial_response);
-	if (inputlen == -1) {
-	    prot_printf(popd_out, "-ERR Invalid base64 string\r\n");
-	    mech->free_state(state);
-	    return;
-	}
-	r = mech->auth(state, inputlen, input, &outputlen, &output, &reply);
+    /* get the userid from SASL --- already canonicalized from
+     * mysasl_authproc()
+     */
+
+    sasl_result = sasl_getprop(popd_saslconn, SASL_USERNAME,
+			     (void **) &popd_userid);
+    if (sasl_result!=SASL_OK)
+    {
+      prot_printf(popd_out, "-ERR weird SASL error %d getting SASL_USERNAME\r\n", 
+		  sasl_result);
+      return;
     }
 
-    while (r == 0) {
-	printauthready(outputlen, output);
-	inputlen = readbase64string(&input);
-	if (inputlen == -1) {
-	    prot_printf(popd_out, "-ERR Invalid base64 string\r\n");
-	    mech->free_state(state);
-	    return;
-	}
-	r = mech->auth(state, inputlen, input, &outputlen, &output, &reply);
-    }
-    
-    if (r != SASL_DONE) {
-	mech->free_state(state);
-	if (reply) {
-	    syslog(LOG_NOTICE, "badlogin: %s %s %s",
-		   popd_clienthost, authtype, reply);
-	}
-	sleep(3);
-	prot_printf(popd_out, "-ERR Invalid login\r\n");
-	return;
+    syslog(LOG_NOTICE, "login: %s %s %s sasl code=%d", popd_clienthost, popd_userid,
+	   authtype, sasl_result);
+
+    /* xxx here */
+    if (openinbox()==0)
+    {
+
+      proc_register("pop3d", popd_clienthost, popd_userid, popd_mailbox->name);    
+
+      syslog(LOG_NOTICE, "login: %s %s %s %s", popd_clienthost, popd_userid,
+	     authtype, "User logged in");
+
+
+      prot_setsasl(popd_in,  popd_saslconn);
+      prot_setsasl(popd_out, popd_saslconn);
+
     }
 
-    mech->query_state(state, &user, &protlevel, &encodefunc, &decodefunc,
-		      &maxplain);
-
-    canon_user = auth_canonifyid(user);
-    if (!canon_user || strchr(canon_user, '.') ||
-	strlen(canon_user) + 6 > MAX_MAILBOX_PATH) {
-	syslog(LOG_NOTICE, "badlogin: %s %s %s bad userid",
-	       popd_clienthost, authtype, beautify_string(user));
-	mech->free_state(state);
-	prot_printf(popd_out, "-ERR Invalid user\r\n");
-	return;
-    }
-
-    popd_userid = xstrdup(canon_user);
-
-    syslog(LOG_NOTICE, "login: %s %s %s %s", popd_clienthost, canon_user,
-	   authtype, reply ? reply : "");
-
-    if (openinbox() == 0 && (encodefunc || decodefunc)) {
-	prot_setfunc(popd_in, decodefunc, state, 0);
-	prot_setfunc(popd_out, encodefunc, state, maxplain);
-    }
-    else {
-	mech->free_state(state);
-    }
 }
+    
 
 /*
  * Complete the login process by opening and locking the user's inbox
