@@ -1,5 +1,5 @@
 /* proc.c -- Server process registry
- $Id: proc.c,v 1.11 1998/05/15 21:49:40 neplokh Exp $
+ $Id: proc.c,v 1.14 1999/10/02 00:43:07 leg Exp $
  
  # Copyright 1998 Carnegie Mellon University
  # 
@@ -27,10 +27,13 @@
  *
  */
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <syslog.h>
 
 #include "config.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "xmalloc.h"
 
 #define FNAME_PROCDIR "/proc/"
@@ -38,7 +41,7 @@
 static char *procfname = 0;
 static FILE *procfile = 0;
 
-proc_register(progname, clienthost, userid, mailbox)
+int proc_register(progname, clienthost, userid, mailbox)
 char *progname;
 char *clienthost;
 char *userid;
@@ -55,7 +58,7 @@ char *mailbox;
 	procfile = fopen(procfname, "w+");
 	if (!procfile) {
 	    syslog(LOG_ERR, "IOERROR: creating %s: %m", procfname);
-	    fatal("can't write proc file", EX_IOERR);
+	    fatal("can't write proc file", EC_IOERR);
 	}
     }
 
@@ -77,11 +80,12 @@ char *mailbox;
     return 0;
 }
 
-proc_cleanup()
+void proc_cleanup()
 {
     if (procfname) {
 	fclose(procfile);
 	unlink(procfname);
-	procfname = 0;
+	free(procfname);
+	procfname = NULL;
     }
 }
