@@ -25,7 +25,7 @@
  *  tech-transfer@andrew.cmu.edu
  */
 
-/* $Id: reconstruct.c,v 1.37 1998/08/07 06:52:04 tjs Exp $ */
+/* $Id: reconstruct.c,v 1.42 1999/08/20 19:05:30 leg Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -59,7 +59,7 @@
 #include "bsearch.h"
 #include "imparse.h"
 #include "config.h"
-#include "sysexits.h"
+#include "exitcodes.h"
 #include "imap_err.h"
 #include "mailbox.h"
 #include "message.h"
@@ -87,7 +87,7 @@ char **argv;
 
     config_init("reconstruct");
 
-    if (geteuid() == 0) fatal("must run as the Cyrus user", EX_USAGE);
+    if (geteuid() == 0) fatal("must run as the Cyrus user", EC_USAGE);
 
     /* Ensure we're up-to-date on the index file format */
     assert(INDEX_HEADER_SIZE == (OFFSET_UIDVALIDITY+4));
@@ -140,7 +140,7 @@ usage()
 {
     fprintf(stderr, "usage: reconstruct [-r] mailbox...\n");
     fprintf(stderr, "       reconstruct -m\n");
-    exit(EX_USAGE);
+    exit(EC_USAGE);
 }    
 
 int compare_uid(a, b)
@@ -645,6 +645,34 @@ char *partition;
     todo_tail = &newentry->next;
 }
 
+void
+todo_append_hashed(char *name, char *path, char *partition)
+{
+    DIR *dirp;
+    struct dirent *dirent;
+
+    dirp = opendir(path);
+    if (!dirp) {
+	fprintf(stderr, "reconstruct: couldn't open partition %s\n", name);
+    } else while (dirent = readdir(dirp)) {
+	struct todo *newentry;
+
+	if (strchr(dirent->d_name, '.')) {
+	    continue;
+	}
+
+	newentry = (struct todo *)xmalloc(sizeof(struct todo));
+	newentry->name = xstrdup(name);
+	newentry->path = xmalloc(strlen(path) +
+				 strlen(dirent->d_name) + 2);
+	sprintf(newentry->path, "%s/%s", path, dirent->d_name);
+	newentry->partition = partition;
+	newentry->next = 0;
+	*todo_tail = newentry;
+	todo_tail = &newentry->next;
+    }
+}
+
 char *cleanacl(acl, mboxname)
 char *acl;
 char *mboxname;
@@ -753,19 +781,13 @@ do_mboxlist()
 	}
 	
 	/* Check mailbox exists */
-	sprintf(pathresult, "%s/%s", root, mboxname);
-	for (p = pathresult + strlen(root); *p; p++) {
-	    if (*p == '.') *p = '/';
-	}
+	mailbox_hash_mbox(pathresult, root, mboxname);
 	r = mailbox_open_header_path(mboxname, pathresult, "", 0,
 				     &mailbox, 1);
 	if (r) {
 	    /* Try lowercasing mailbox name */
 	    lcase(mboxname);
-	    sprintf(pathresult, "%s/%s", root, mboxname);
-	    for (p = pathresult + strlen(root); *p; p++) {
-		if (*p == '.') *p = '/';
-	    }
+	    mailbox_hash_mbox(pathresult, root, mboxname);
 
 	    r = mailbox_open_header_path(mboxname, pathresult, "", 0,
 					 &mailbox, 1);
@@ -788,8 +810,12 @@ do_mboxlist()
     }
     
     /* Enqueue each partition directory for scanning */
-    config_scanpartition(todo_append);
-    
+    if (config_hashimapspool) {
+	config_scanpartition(todo_append_hashed);
+    } else {
+	config_scanpartition(todo_append);
+    }
+
     /* Process each directory in queue */
     while (todo_head) {
 	isnewspartition = (strcmp(todo_head->partition, "news") == 0);
@@ -873,9 +899,7 @@ do_mboxlist()
     exit(0);
 }
 
-fatal(s, code)
-char *s;
-int code;
+void fatal(const char* s, int code)
 {
     fprintf(stderr, "reconstruct: %s\n", s);
     exit(code);
