@@ -1,5 +1,5 @@
 /* assert.c -- handle assertion failures
- $Id: assert.c,v 1.9 1998/05/15 21:50:33 neplokh Exp $
+ $Id: assert.c,v 1.11 1999/09/30 07:30:06 leg Exp $
  
  #        Copyright 1998 by Carnegie Mellon University
  #
@@ -24,7 +24,8 @@
  */
 #include <stdio.h>
 
-#include "sysexits.h"
+#include "xmalloc.h"
+#include "exitcodes.h"
 #include "assert.h"
 
 int assertionfailed(file, line, expr)
@@ -36,5 +37,5 @@ const char *expr;
 
     sprintf(buf, "Internal error: assertion failed: %s: %d%s%s",
 	    file, line, expr ? ": " : "", expr ? expr : "");
-    fatal(buf, EX_SOFTWARE);
+    fatal(buf, EC_SOFTWARE);
 }
