@@ -1,5 +1,5 @@
 /* auth_krb.c -- Kerberos authorization
- $Id: auth_krb.c,v 1.27 1998/05/15 21:50:43 neplokh Exp $
+ $Id: auth_krb.c,v 1.29 1999/11/16 01:02:12 leg Exp $
  
  #        Copyright 1998 by Carnegie Mellon University
  #
@@ -30,13 +30,18 @@
 #include <sys/types.h>
 
 #include <krb.h>
+#ifdef HAVE_LIBDB
 #ifdef HAVE_DB_185_H
 #include <db_185.h>
 #else
 #include <db.h>
 #endif
+#else
+#include <ndbm.h>
+#endif
 #include <krb.h>
 
+#include "xmalloc.h"
 #include "auth.h"
 
 #ifndef KRB_MAPNAME
