@@ -1,5 +1,5 @@
 /* auth_krb_pts.c -- Kerberos authorization with AFS PTServer groups
- $Id: auth_krb_pts.c,v 1.23 1998/08/16 10:48:03 wcw Exp $
+ $Id: auth_krb_pts.c,v 1.29 1999/09/30 07:30:18 leg Exp $
  
  #        Copyright 1998 by Carnegie Mellon University
  #
@@ -24,6 +24,8 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -35,10 +37,12 @@
 #else
 #include <db.h>
 #endif
+
 #include <krb.h>
 
-#include "auth.h"
 #include "auth_krb_pts.h"
+#include "auth.h"
+#include "lock.h"
 #include "retry.h"
 #include "xmalloc.h"
 
@@ -233,7 +237,7 @@ const char *identifier;
     }
     
     aname[0] = inst[0] = realm[0] = '\0';
-    if (kname_parse(aname, inst, realm, identifier) != 0) {
+    if (kname_parse(aname, inst, realm, (char *) identifier) != 0) {
         return 0;
     }
     
@@ -303,38 +307,36 @@ const char *cacheid;
     struct auth_state *newstate;
     DBT key, dataheader,datalist;
     char keydata[PTS_DB_KEYSIZE];
-    int fd,rc,xpid;
     char fnamebuf[1024];
     DB *ptdb;
     HASHINFO info;
     ptluser us;
-    int forktries = 0;
     int s;
     struct sockaddr_un srvaddr;
     int r;
+    int fd, rc;
     struct iovec iov[10];
     static char response[1024];
     int start, n;
-    
+
     identifier = auth_canonifyid(identifier);
     if (!identifier) return 0;
 
     newstate = (struct auth_state *)xmalloc(sizeof(struct auth_state));
+    (void)memset(newstate, 0, sizeof(struct auth_state));
 
-    newstate->aname[0] = newstate->inst[0] = newstate->realm[0] = '\0';
-    newstate->ngroups = 0;
-    newstate->groups = 0;
+    kname_parse(newstate->aname, newstate->inst, newstate->realm, 
+		(char *) identifier);
+    (void)strcpy(newstate->userid, identifier);
 
-    kname_parse(newstate->aname, newstate->inst, newstate->realm, identifier);
-    if (strcmp(newstate->userid, "anyone") == 0) return newstate;
-    strcpy(newstate->userid, identifier);
+    if (strcmp(identifier, "anyone") == 0) return newstate;
 
     (void)memset(&info, 0, sizeof(info));
     (void)memset(&key, 0, sizeof(key));
     key.data = keydata;
     key.size = PTS_DB_KEYSIZE;
-    strcpy(fnamebuf, STATEDIR);
-    strcat(fnamebuf, PTS_DBLOCK);
+    (void)strcpy(fnamebuf, STATEDIR);
+    (void)strcat(fnamebuf, PTS_DBLOCK);
     fd = open(fnamebuf, O_CREAT|O_TRUNC|O_RDWR, 0664);
     if (fd == -1) {
         syslog(LOG_ERR, "IOERROR: creating lock file %s: %m", fnamebuf);
@@ -344,9 +346,13 @@ const char *cacheid;
         syslog(LOG_ERR, "IOERROR: locking lock file %s: %m", fnamebuf);
         return newstate;
     }
-    strcpy(fnamebuf, STATEDIR);
-    strcat(fnamebuf, PTS_DBFIL);
+    (void)strcpy(fnamebuf, STATEDIR);
+    (void)strcat(fnamebuf, PTS_DBFIL);
     ptdb = dbopen(fnamebuf, O_RDONLY, 0, DB_HASH, &info);
+
+    (void)memset(&dataheader, 0, sizeof(dataheader));
+    (void)memset(&datalist, 0, sizeof(datalist));
+
     if (!ptdb) {
 	if (errno == ENOENT) {
 	    /*
@@ -373,8 +379,8 @@ const char *cacheid;
 		 * Write a record to the database, so that the database
 		 * header will be written out
 		 */
-		memset(key.data, 0, key.size);
-		strcpy(key.data, "DUMMYREC");
+	        (void)memset(key.data, 0, key.size);
+		(void)strcpy(key.data, "DUMMYREC");
 		dataheader.size = 5;
 		dataheader.data = "NULL";
 		if (PUT(ptdb, &key, &dataheader, 0) < 0) {
@@ -408,19 +414,19 @@ const char *cacheid;
     }
     if (cacheid) {
       /* this should be the session key + the userid */
-        memset(keydata, 0, key.size);
-        memcpy(keydata, cacheid, 16); /* why 16? see sasl_krb_server.c */
+        (void)memset(keydata, 0, key.size);
+        (void)memcpy(keydata, cacheid, 16); /* why 16? see sasl_krb_server.c */
 	/* toss on userid to further uniquify */
 	if ((strlen(identifier) + 16)  < PTS_DB_KEYSIZE) {
-	  memcpy(keydata+16, identifier, strlen(identifier)); 
+	  (void)memcpy(keydata+16, identifier, strlen(identifier)); 
 	} else {
-	  memcpy(keydata+16, identifier, PTS_DB_KEYSIZE-16);
+	  (void)memcpy(keydata+16, identifier, PTS_DB_KEYSIZE-16);
 	}
     } /* cacheid */
     else {
       /* this is just the userid */
-        memset(keydata, 0, key.size);
-        strncpy(keydata, identifier, PR_MAXNAMELEN);
+        (void)memset(keydata, 0, key.size);
+        (void)strncpy(keydata, identifier, PR_MAXNAMELEN);
     }
     /* Fetch and process the header record for the user, if any */
     keydata[PTS_DB_HOFFSET] = 'H';
@@ -441,7 +447,7 @@ const char *cacheid;
             return newstate;
         }
         /* make sure the record is aligned */
-        memcpy(&us, dataheader.data, sizeof(ptluser));
+        (void)memcpy(&us, dataheader.data, sizeof(ptluser));
     }
     if (rc || (!cacheid && us.cached < time(0) - EXPIRE_TIME)) {
         CLOSE(ptdb);
@@ -450,12 +456,12 @@ const char *cacheid;
         s = socket(AF_UNIX, SOCK_STREAM, 0);
         if (s == -1) return newstate;
         
-	strcpy(fnamebuf, STATEDIR);
-	strcat(fnamebuf, PTS_DBSOCKET);
+	(void)strcpy(fnamebuf, STATEDIR);
+	(void)strcat(fnamebuf, PTS_DBSOCKET);
 
-        memset((char *)&srvaddr, 0, sizeof(srvaddr));
+        (void)memset((char *)&srvaddr, 0, sizeof(srvaddr));
         srvaddr.sun_family = AF_UNIX;
-        strcpy(srvaddr.sun_path, fnamebuf);
+        (void)strcpy(srvaddr.sun_path, fnamebuf);
         r = connect(s, (struct sockaddr *)&srvaddr, sizeof(srvaddr));
         if (r == -1) {
 	    /* *reply = "cannot connect to ptloader server";*/
@@ -485,8 +491,8 @@ const char *cacheid;
 
         /* The database must be re-opened after external modifications, at
            least in db 1.1.85 */
-	strcpy(fnamebuf, STATEDIR);
-	strcat(fnamebuf, PTS_DBLOCK);
+	(void)strcpy(fnamebuf, STATEDIR);
+	(void)strcat(fnamebuf, PTS_DBLOCK);
         fd = open(fnamebuf, O_CREAT|O_TRUNC|O_RDWR, 0664);
         if (fd == -1) {
             syslog(LOG_ERR, "IOERROR: creating lock file %s: %m", fnamebuf);
@@ -496,8 +502,8 @@ const char *cacheid;
             syslog(LOG_ERR, "IOERROR: locking lock file %s: %m", fnamebuf);
             return newstate;
         }
-	strcpy(fnamebuf, STATEDIR);
-	strcat(fnamebuf, PTS_DBFIL);
+	(void)strcpy(fnamebuf, STATEDIR);
+	(void)strcat(fnamebuf, PTS_DBFIL);
         ptdb = dbopen(fnamebuf, O_RDONLY, 0, DB_HASH, &info);
         if (!ptdb) {
             syslog(LOG_ERR, "IOERROR: opening database %s: %m", fnamebuf);
@@ -511,20 +517,17 @@ const char *cacheid;
         keydata[PTS_DB_HOFFSET] = 0;
         if (rc < 0) {
             syslog(LOG_ERR, "IOERROR: reading database: %m");             
-            CLOSE(ptdb);
-            close(fd);
-            return newstate;
+	    goto done;
         }
         /* The record still isn't there, even though the child claimed sucess
          */ 
         if (rc) {
             syslog(LOG_ERR, "ptloader did not add database record for %s",
                    identifier);
-            CLOSE(ptdb);
-            close(fd);
-            return newstate;
+	    goto done;
+
         }
-        memcpy(&us, dataheader.data, dataheader.size);
+        (void)memcpy(&us, dataheader.data, dataheader.size);
     }
     /*
      * We assume cache keys will be unique. This will catch duplicates if they
@@ -534,9 +537,7 @@ const char *cacheid;
         syslog(LOG_ERR,
                "Internal error: Fetched record for user %s was for user %s: key not unique",
                identifier, us.user);
-        CLOSE(ptdb);
-        close(fd);      
-        return newstate;
+	goto done;
     }
     /*
      * now get the actual data from the database. this will be a contiguous
@@ -544,23 +545,35 @@ const char *cacheid;
      */
     keydata[PTS_DB_HOFFSET] = 'D';
     rc = GET(ptdb, &key, &datalist, 0);
-    CLOSE(ptdb);    
-    close(fd);
     if (rc < 0) {
         syslog(LOG_ERR, "IOERROR: reading database %s: %m", fnamebuf);
-        return newstate;
+	goto done;
     }
     if (rc) {
         syslog(LOG_ERR,
                "Database %s inconsistent: header record found, data record missing", fnamebuf);
-        return newstate;
+	goto done;
     }
+
     newstate->ngroups = us.ngroups;
-    if (newstate->ngroups) {
-        newstate->groups = (char (*)[PR_MAXNAMELEN])xmalloc(newstate->ngroups *
-							 PR_MAXNAMELEN); 
-        memcpy(newstate->groups, datalist.data, newstate->ngroups*PR_MAXNAMELEN);
+
+#if 0
+    if (newstate->ngroups * PR_MAXNAMELEN < datalist.size) {
+	syslog(LOG_ERR,
+	       "Database %s inconsistent: ngroups(%d) * PR_MAXNAMELEN(%d) < datalist.size(%d)",
+	       fnamebuf, newstate->ngroups, PR_MAXNAMELEN, datalist.size);
     }
+#endif
+
+    if (newstate->ngroups) {
+      newstate->groups = (char (*)[PR_MAXNAMELEN])xmalloc(newstate->ngroups *
+							  PR_MAXNAMELEN); 
+      (void)memcpy(newstate->groups, datalist.data, newstate->ngroups*PR_MAXNAMELEN);
+    }
+
+ done:
+    CLOSE(ptdb);    
+    close(fd);
     return newstate;
 }
 
