@@ -1,5 +1,5 @@
 /* imclient.c -- Streaming IMxP client library
- $Id: imclient.c,v 1.25 1998/05/15 21:51:36 neplokh Exp $
+ $Id: imclient.c,v 1.45 1999/12/23 21:03:19 leg Exp $
  
  #        Copyright 1998 by Carnegie Mellon University
  #
@@ -44,8 +44,18 @@
 #include <sys/select.h>
 #endif
 
-#include "sasl.h"
-#include "sysexits.h"
+#include <sasl.h>
+
+#ifdef HAVE_SSL
+#include <openssl/lhash.h>
+#include <openssl/bn.h>
+#include <openssl/err.h>
+#include <openssl/pem.h>
+#include <openssl/x509.h>
+#include <openssl/ssl.h>
+#endif /* HAVE_SSL */
+
+#include "exitcodes.h"
 #include "xmalloc.h"
 #include "imparse.h"
 #include "imclient.h"
@@ -85,23 +95,18 @@ struct imclient {
     char *outptr;
     int outleft;
     char *outstart;
-    char outcryptbuf[IMCLIENT_BUFSIZE+4];
-    char *outcryptstart;
-    int outcryptlen;
 
     /* Replies being received from server */
     char *replybuf;
     char *replystart;
     int replyliteralleft;
     int replylen;
-    int replycryptstart;
-    int replycryptend;
     int alloc_replybuf;
     
     /* Protection mechanism data */
-    struct sasl_client *mech;
+  /*    struct sasl_client *mech;
     sasl_encodefunc_t *encodefunc;
-    sasl_decodefunc_t *decodefunc;
+    sasl_decodefunc_t *decodefunc;*/
     void *state;
     int maxplain;
     
@@ -119,6 +124,15 @@ struct imclient {
     int callback_num;
     int callback_alloc;
     struct imclient_callback *callback;
+
+  sasl_conn_t *saslconn;
+  int saslcompleted;
+
+#ifdef HAVE_SSL
+  SSL_CTX *tls_ctx;
+  SSL *tls_conn;
+  int tls_on; /* wheather we are under a layer or not */
+#endif /* HAVE_SSL */
 };
 
 /*
@@ -157,6 +171,19 @@ static void imclient_writebase64 P((struct imclient *imclient,
 static void imclient_eof P((struct imclient *imclient));
 static int imclient_decodebase64 P((char *input));
 
+/* callbacks we support */
+static sasl_callback_t callbacks[] = {
+  {
+    SASL_CB_USER, NULL, NULL
+  }, {
+    SASL_CB_AUTHNAME, NULL, NULL
+  }, {
+    SASL_CB_PASS, NULL, NULL    
+  }, {
+    SASL_CB_LIST_END, NULL, NULL
+  }
+};
+
 /*
  * Connect to server on 'host'.  Optional 'port' specifies the service
  * to use.  On success, returns zero and fills in the pointer pointed
@@ -174,6 +201,7 @@ const char *port;
     struct servent *sp;
     struct sockaddr_in addr;
     static struct imclient zeroimclient;
+    int saslresult;
 
     hp = gethostbyname(host);
     if (!hp) return -1;
@@ -197,10 +225,12 @@ const char *port;
     if (connect(s, (struct sockaddr *)&addr, sizeof(addr)) == -1) {
 	return errno;
     }
-    nonblock(s, 1);
+    /*    nonblock(s, 1); */
     *imclient = (struct imclient *)xmalloc(sizeof(struct imclient));
     **imclient = zeroimclient;
     (*imclient)->fd = s;
+    (*imclient)->saslconn = NULL;
+    (*imclient)->saslcompleted = 0;
     (*imclient)->servername = xstrdup(hp->h_name);
     (*imclient)->outptr = (*imclient)->outstart = (*imclient)->outbuf;
     (*imclient)->outleft = (*imclient)->maxplain = sizeof((*imclient)->outbuf);
@@ -212,6 +242,25 @@ const char *port;
 		 "BYE", CALLBACK_NOLITERAL, (imclient_proc_t *)0, (void *)0,
 		 (char *)0);
 
+#ifdef HAVE_SSL
+    (*imclient)->tls_ctx=NULL;
+    (*imclient)->tls_conn=NULL;
+    (*imclient)->tls_on=0;
+#endif /* HAVE_SSL */
+
+
+  /* attempt to start sasl */
+  saslresult=sasl_client_init(callbacks);
+  if (saslresult!=SASL_OK) return 1;
+
+  /* client new connection */
+  saslresult=sasl_client_new("imap", /* xxx ideally this should be configurable */
+			     (*imclient)->servername,
+			     NULL,
+			     0,
+			     &((*imclient)->saslconn));
+  if (saslresult!=SASL_OK) return 1;
+
     return 0;
 }
 
@@ -228,7 +277,7 @@ struct imclient *imclient;
     close(imclient->fd);
     free(imclient->servername);
     if (imclient->replybuf) free(imclient->replybuf);
-    if (imclient->state) imclient->mech->free_state(imclient->state);
+    /*    if (imclient->state) imclient->mech->free_state(imclient->state);*/
     
     for (i = 0; i < imclient->callback_num; i++) {
 	free(imclient->callback[i].keyword);
@@ -468,7 +517,7 @@ va_dcl
 
 	default:
 	    fatal("internal error: invalid format specifier in imclient_send",
-		  EX_SOFTWARE);
+		  EC_SOFTWARE);
 	}
 	fmt = percent + 1;
     }
@@ -589,9 +638,25 @@ int len;
     int keywordlen;
     int keywordindex;
     struct imclient_cmdcallback **cmdcb, *cmdcbtemp;
+    char *plainbuf;
+    int plainlen;
+    int freeplain;
+    
+    if (imclient->saslcompleted == 1) {
+	/* decrypt what we have */
+	if (sasl_decode(imclient->saslconn, buf, len,
+			&plainbuf, &plainlen) != SASL_OK) {
+	    (void) shutdown(imclient->fd, 0);
+	}
+	freeplain = 1;
+    } else {
+	plainbuf = buf;
+	plainlen = len;
+	freeplain = 0;
+    }
 
     /* Ensure replybuf has enough space to take the input */
-    if (imclient->replycryptend + len >= imclient->alloc_replybuf) {
+    if (imclient->replylen + plainlen >= imclient->alloc_replybuf) {
 	/* If there is unused space at the front, move the plaintext there */
 	if (imclient->replystart != imclient->replybuf) {
 	    imclient->replylen -= imclient->replystart - imclient->replybuf;
@@ -599,85 +664,37 @@ int len;
 		    imclient->replylen);
 	    imclient->replystart = imclient->replybuf;
 	}
-	/* If unused space between plaintext and crypttext, move crypttext */
-	if (imclient->replycryptstart > imclient->replylen) {
-	    if (imclient->replycryptend - imclient->replycryptstart) {
-		memmove(imclient->replybuf + imclient->replylen,
-			imclient->replybuf + imclient->replycryptstart,
-			imclient->replycryptend - imclient->replycryptstart);
-	    }
-	    imclient->replycryptend -=
-	      imclient->replycryptstart - imclient->replylen;
-	    imclient->replycryptstart = imclient->replylen;
 
-	    /* Shrink the reply buffer if it's too large */
-	    if (imclient->replycryptend + len + REPLYSHRINK <
+	/* Shrink the reply buffer if it's too large */
+	if (imclient->replylen + plainlen + REPLYSHRINK <
 		imclient->alloc_replybuf) {
-		imclient->alloc_replybuf = imclient->replycryptend + len
+		imclient->alloc_replybuf = imclient->replylen + plainlen
 		  + REPLYSHRINK;
 		imclient->replybuf = xrealloc(imclient->replybuf,
 					      imclient->alloc_replybuf);
 		imclient->replystart = imclient->replybuf;
 	    }
-	}
 
 	/* If there still isn't enough room, grow the buffer */
-	if (imclient->replycryptend + len >= imclient->alloc_replybuf) {
+	if (imclient->replylen + plainlen >= imclient->alloc_replybuf) {
 	    imclient->alloc_replybuf =
-	      imclient->replycryptend + len + REPLYSLACK;
+	      imclient->replylen + plainlen + REPLYSLACK;
 	    imclient->replybuf = xrealloc(imclient->replybuf,
 					  imclient->alloc_replybuf);
 	    imclient->replystart = imclient->replybuf;
 	}
     }
-
-    /* Copy the data to the buffer and NUL-terminate it */
-    memcpy(imclient->replybuf + imclient->replycryptend, buf, len);
-    imclient->replycryptend += len;
+    
 
     /* Remember where new data starts */
     parsed = imclient->replylen;
 
-    /* Decrypt the data */
-    if (imclient->decodefunc) {
-	char lenbuf[4];
-	int toklen, len;
-	char *plainptr;
-	int plainlen;
+    /* Copy the data to the buffer and NUL-terminate it */
+    memcpy(imclient->replybuf + imclient->replylen, plainbuf, plainlen);
+    imclient->replylen += plainlen;
 
-	for (;;) {
-	    /* Make sure we have an entire token */
-	    len = imclient->replycryptend - imclient->replycryptstart - 4;
-	    if (len < 0) break;
-	    memcpy(lenbuf, imclient->replybuf + imclient->replycryptstart, 4);
-	    toklen = ntohl(*(int *)lenbuf);
-	    if (toklen > IMCLIENT_BUFSIZE) {
-		/* XXX report error: token too long */
-		(void) shutdown(imclient->fd, 0);
-		break;
-	    }
-	    if (len < toklen) break;
-
-	    /* Decode the crypttext token */
-	    if (imclient->decodefunc(imclient->state,
-		     imclient->replybuf + imclient->replycryptstart + 4,
-				     toklen, &plainptr, &plainlen)) {
-		/* XXX report decode error */
-		(void) shutdown(imclient->fd, 0);
-		break;
-	    }
-
-	    /* Copy the plaintext out, done with crypttext */
-	    memmove(imclient->replybuf + imclient->replylen,
-		    plainptr, plainlen);
-	    imclient->replylen += plainlen;
-	    imclient->replycryptstart += 4+toklen;
-	}
-    }
-    else {
-	/* No decryption necessary */
-	imclient->replylen = imclient->replycryptstart =
-	  imclient->replycryptend;
+    if (freeplain && plainlen) {
+	free(plainbuf);
     }
 
     /* Process the new data */
@@ -714,6 +731,7 @@ int len;
 	    if (imclient->readytag) {
 		imclient->readytag = 0;
 		imclient->readytxt = p+2;
+		*(endreply-1) = '\0';
 	    }
 	    else {
 		/* XXX Got junk from the server */
@@ -918,8 +936,7 @@ int *fd;
 int *wanttowrite;
 {
     *fd = imclient->fd;
-    *wanttowrite = imclient->outcryptlen ||
-	imclient->outptr - imclient->outstart;
+    *wanttowrite = imclient->outptr - imclient->outstart;
 }
 
 /*
@@ -937,41 +954,56 @@ struct imclient *imclient;
     for (;;) {
 	writelen = imclient->outptr - imclient->outstart;
 
-	if (imclient->encodefunc) {
-	    /*
-	     * If output crypttext buffer empty and have output plaintext,
-	     * encode the plaintext, moving it to the crypttext buffer.
-	     */
-	    if (!imclient->outcryptlen && writelen) {
-		imclient->outcryptstart = imclient->outcryptbuf;
-		if (imclient->encodefunc(imclient->state, imclient->outstart,
-					 writelen, imclient->outcryptbuf+4,
-					 &n)) {
-		    /* XXX encoding error */
-		    n = 0;
-		}
-		*(int *)imclient->outcryptbuf = htonl(n);
-		imclient->outcryptlen = n + 4;
-		imclient->outstart += writelen;
-	    }
+	if ((imclient->saslcompleted==1) && (writelen>0)) {
+	    unsigned int cryptlen=0;
+	    char *cryptptr=NULL;
+
+	  if (sasl_encode(imclient->saslconn, imclient->outstart, writelen,
+			  &cryptptr,&cryptlen)!=SASL_OK)
+	  {
+	      /* XXX encoding error */
+	      n=0;
+	  }
+
+#ifdef HAVE_SSL
+	  if (imclient->tls_on==1)
+	  {
+	    n = SSL_write(imclient->tls_conn, cryptptr, cryptlen);
+	  } else {
+	    n = write(imclient->fd, cryptptr, cryptlen);
+	  }
+#else  /* HAVE_SSL */
+	  n = write(imclient->fd, cryptptr,
+		    cryptlen);
+#endif /* HAVE_SSL */
+	  	  
+	  if (n > 0) {	    
+	    free(cryptptr);
+	    imclient->outstart += writelen;
+	    return;
+	  }
+
+
+	  /* XXX Also EPIPE & the like? */
+	  /* Make sure we select() for writing */
 
-	    /* If have output crypttext, write it out */
-	    if (imclient->outcryptlen) {
-		n = write(imclient->fd, imclient->outcryptstart,
-			  imclient->outcryptlen);
-		if (n > 0) {
-		    imclient->outcryptstart += n;
-		    imclient->outcryptlen -= n;
-		    return;
-		}
-		/* XXX Also EPIPE & the like? */
-		/* Make sure we select() for writing */
-		writelen = imclient->outcryptlen;
-	    }
 	}
 	else if (writelen) {
-	    /* No protection mechanism, just write the plaintext */
+
+	  /* No protection mechanism, just write the plaintext */
+
+#ifdef HAVE_SSL
+	  if (imclient->tls_on==1)
+	  {
+	    n = SSL_write(imclient->tls_conn, imclient->outstart, writelen);
+	  } else {
 	    n = write(imclient->fd, imclient->outstart, writelen);
+	  }
+#else  /* HAVE_SSL */
+	  n = write(imclient->fd, imclient->outstart, writelen);
+#endif /* HAVE_SSL */
+
+
 	    if (n > 0) {
 		imclient->outstart += n;
 		return;
@@ -979,15 +1011,31 @@ struct imclient *imclient;
 	    /* XXX Also EPIPE & the like? */
 	}
 
-	n = read(imclient->fd, buf, sizeof(buf));
-	if (n >= 0) {
+	if (FD_ISSET(imclient->fd, &rfds))
+	{
+#ifdef HAVE_SSL	  
+	  /* just do a SSL read instead if we're under a tls layer */
+	  if (imclient->tls_on==1)
+	  {
+	    n = SSL_read(imclient->tls_conn, buf, sizeof(buf));
+
+	  } else {
+	    n = read(imclient->fd, buf, sizeof(buf));
+	  }
+
+#else  /* HAVE_SSL */
+	  n = read(imclient->fd, buf, sizeof(buf));
+#endif /* HAVE_SSL */
+
+	  if (n >= 0) {
 	    if (n == 0) {
-		imclient_eof(imclient);
+	      imclient_eof(imclient);
 	    }
 	    else {
-		imclient_input(imclient, buf, n);
+	      imclient_input(imclient, buf, n);
 	    }
 	    return;
+	  }
 	}
 
 	FD_ZERO(&rfds);
@@ -1013,35 +1061,9 @@ struct imclient *imclient;
 void *rock;
 struct imclient_reply *reply;
 {
-    char *userid;
-    int protlevel;
     struct authresult *result = (struct authresult *)rock;
 
     if (!strcmp(reply->keyword, "OK")) {
-	/* Check for premature command completion */
-	if (result->r != SASL_DONE) {
-	    result->replytype = replytype_prematureok;
-	    return;
-	}
-
-	/* Query for a negotiated protection mechanism */
-	(imclient->mech)->query_state(imclient->state, &userid, &protlevel,
-				      &imclient->encodefunc,
-				      &imclient->decodefunc,
-				      &imclient->maxplain);
-
-	if (imclient->encodefunc || imclient->decodefunc) {
-	    /* Protection starts right after tagged OK command */
-	    imclient->replylen = imclient->replycryptstart =
-	      imclient->replystart - imclient->replybuf;
-	}
-	else {
-	    /* No protection mechanism, free the authentication state */
-	    (imclient->mech)->free_state(imclient->state);
-	    imclient->state = 0;
-	    imclient->maxplain = sizeof(imclient->outbuf);
-	}
-
 	result->replytype = replytype_ok;
     }
     else if (!strcmp(reply->keyword, "NO")) {
@@ -1050,119 +1072,223 @@ struct imclient_reply *reply;
     else result->replytype = replytype_bad;
 }
 
-/*
- * Authenticate the connection 'imclient' using one of the mechanisms
- * in 'availmech'.  'user', if non-null, specifies the user to authenticate
- * as.  'protallowed' is a bitmask of permissible protection mechanisms.
- */
-int
-imclient_authenticate(imclient, availmech, service, user, protallowed)
+/* Command completion for starttls */
+static void 
+tlsresult(imclient, rock, reply)
 struct imclient *imclient;
-const char *service;
-struct sasl_client **availmech;
-const char *user;
-int protallowed;
+void *rock;
+struct imclient_reply *reply;
 {
-    struct sasl_client **mech;
-    struct sockaddr localaddr, remoteaddr;
-    int localaddrlen, remoteaddrlen;
-    int gotaddr = 1;
-    struct authresult result;
-    int inputlen, outputlen;
-    char *output;
+    struct authresult *result = (struct authresult *)rock;
 
-    /*
-     * Try to get the connection network addresses, for use by the
-     * protection mechanisms.
-     */
-    localaddrlen = remoteaddrlen = sizeof(localaddr);
-    if (getsockname(imclient->fd, (struct sockaddr *)&localaddr,
-		    &localaddrlen) < 0 ||
-	getpeername(imclient->fd, (struct sockaddr *)&remoteaddr,
-		    &remoteaddrlen) < 0) {
-	gotaddr = 0;
+    if (!strcmp(reply->keyword, "OK")) {
+      result->replytype = replytype_ok;
     }
-
-    /* Try each mechanism in turn */
-    for (mech = availmech; *mech; mech++) {
-	if ((*mech)->start((*mech)->rock, service, imclient->servername,
-			   user, protallowed,
-			   IMCLIENT_BUFSIZE, gotaddr ? &localaddr : 0,
-			   gotaddr ? &remoteaddr : 0, &imclient->state)) {
-	    continue;
-	}
-
-	imclient->mech = *mech;
-	result.r = 0;
-	result.replytype = replytype_inprogress;
-
-	if ((*mech)->can_send_initial_response &&
-	    (imclient->flags & IMCLIENT_CONN_INITIALRESPONSE)) {
-	    if ((*mech)->auth(imclient->state,
-			      0, 0,
-			      &outputlen, &output) == SASL_FAIL) {
-		continue;
-	    }
-	    imclient_send(imclient, authresult, (void *)&result,
-			  "AUTHENTICATE %a %B", (*mech)->auth_type,
-			  outputlen, output);
-	}
-	else {
-	    imclient_send(imclient, authresult, (void *)&result,
-			  "AUTHENTICATE %a", (*mech)->auth_type);
-	}
-
-	for (;;) {
-	    /* Wait for ready response or command completion */
-	    imclient->readytag = imclient->gensym;
-	    while (imclient->readytag) {
-		imclient_processoneevent(imclient);
-	    }
-
-	    /* stop looping on command completion */
-	    if (!imclient->readytxt) break;
-
-	    /*
-	     * Base64-decode the challenge and run through the
-	     * authentication mechanism.
-	     */
-	    inputlen = imclient_decodebase64(imclient->readytxt);
-	    if (inputlen != -1) {
-		result.r = (*mech)->auth(imclient->state,
-					 inputlen, imclient->readytxt,
-					 &outputlen, &output);
-	    }
-	    if (inputlen == -1 || result.r == SASL_FAIL) {
-		/* Abort this authentication exchange */
-		imclient_write(imclient, "*\r\n", 3);
-		continue;
-	    }
-
-	    /* Send our reply to the server */
-	    imclient_writebase64(imclient, output, outputlen);
-	}
-	
-	/* Everything happy */
-	if (result.replytype == replytype_ok) {
-	    return 0;
-	}
-
-	/* Something failed, clean up after ourselves */
-	(*mech)->free_state(imclient->state);
-	imclient->state = 0;
-	imclient->mech = 0;
-
-	/* If we got a premature OK, signal severe error to caller */
-	if (result.replytype == replytype_prematureok) return 2;
-
-	/* If we got BAD, stop trying authentication mechanisms */
-	if (result.r == 0 && result.replytype == replytype_bad) {
-	    return 1;
-	}
+    else if (!strcmp(reply->keyword, "NO")) {
+	result->replytype = replytype_no;
     }
-    return 1;
+    else result->replytype = replytype_bad;
 }
 
+
+static sasl_security_properties_t *make_secprops(int min,int max)
+{
+  sasl_security_properties_t *ret=(sasl_security_properties_t *)
+  malloc(sizeof(sasl_security_properties_t));
+
+  ret->maxbufsize=1024;
+  ret->min_ssf=min;
+  ret->max_ssf=max;
+
+  ret->security_flags=0;
+  ret->property_names=NULL;
+  ret->property_values=NULL;
+
+  return ret;
+}
+
+void interaction (sasl_interact_t *t, char *user)
+{
+  char result[1024];
+
+  if ((t->id == SASL_CB_USER || t->id == SASL_CB_AUTHNAME) 
+            && user && user[0]) {
+      t->len = strlen(user);
+      t->result = xstrdup(user);
+  } else {
+      printf("%s: ", t->prompt);
+      if (t->id == SASL_CB_PASS) {
+	  char *ptr = getpass("");
+	  strncpy(result, ptr, sizeof(result));
+      } else {
+	  fgets(result, sizeof(result), stdin);
+	  result[strlen(result) - 1] = '\0';
+      }
+
+      t->len = strlen(result);
+      t->result = (char *) xmalloc(t->len+1);
+      memset(t->result, 0, t->len+1);
+      memcpy((char *) t->result, result, t->len);
+  }
+}
+
+void fillin_interactions(sasl_interact_t *tlist, char *user)
+{
+  while (tlist->id!=SASL_CB_LIST_END)
+  {
+    interaction(tlist, user);
+    tlist++;
+  }
+
+}
+
+
+
+/*
+ * Params:
+ *  mechlist: list of mechanisms seperated by spaces
+ *
+ * Returns:
+ *  0 - sucess
+ *  1 - failure
+ *  2 - severe failure?
+ */
+
+int imclient_authenticate(struct imclient *imclient, 
+			  char *mechlist, 
+			  char *service, 
+			  char *user, 
+			  int minssf, 
+			  int maxssf)
+{
+  int saslresult;
+  sasl_security_properties_t *secprops=NULL;
+  int addrsize=sizeof(struct sockaddr_in);
+  struct sockaddr_in *saddr_l=malloc(sizeof(struct sockaddr_in));
+  struct sockaddr_in *saddr_r=malloc(sizeof(struct sockaddr_in));
+  sasl_interact_t *client_interact=NULL;
+  char *out;
+  unsigned int outlen;
+  int inlen;
+  const char *mechusing;
+  struct authresult result;
+
+
+
+  /*******
+   * Now set the SASL properties
+   *******/
+
+  secprops=make_secprops(minssf,maxssf);
+  if (secprops==NULL) return 1;
+
+  saslresult=sasl_setprop(imclient->saslconn, SASL_SEC_PROPS, secprops);
+  if (saslresult!=SASL_OK) return 1;
+  free(secprops);
+
+  if (getpeername(imclient->fd,(struct sockaddr *)saddr_r,&addrsize)!=0)
+    return 1;
+
+  /*  saddr_r->sin_port=htons(saddr_r->sin_port);	*/
+  saslresult=sasl_setprop(imclient->saslconn, SASL_IP_REMOTE, saddr_r);
+  if (saslresult!=SASL_OK) return 1;
+
+  addrsize=sizeof(struct sockaddr_in);
+  if (getsockname(imclient->fd,(struct sockaddr *)saddr_l,&addrsize)!=0)
+    return 1;
+
+  /*  saddr_l->sin_port=htons(saddr_l->sin_port);	*/
+  saslresult=sasl_setprop(imclient->saslconn,   SASL_IP_LOCAL, saddr_l);
+  if (saslresult!=SASL_OK) return 1;
+
+  free(saddr_l);
+  free(saddr_r);
+
+  /********
+   * SASL is setup. Now try the actual authentication
+   ********/
+    
+  saslresult=SASL_INTERACT;
+
+  /* call sasl client start */
+  while (saslresult==SASL_INTERACT)
+  {
+    saslresult=sasl_client_start(imclient->saslconn, mechlist,
+				 NULL, &client_interact,
+				 &out, &outlen,
+				 &mechusing);
+    if (saslresult==SASL_INTERACT) {
+	fillin_interactions(client_interact, user); /* fill in prompts */
+    }
+  }
+
+  if ((saslresult!=SASL_OK) && (saslresult!=SASL_CONTINUE)) return saslresult;
+
+  imclient_send(imclient, authresult, (void *)&result,
+		"AUTHENTICATE %a", mechusing);
+
+  while (1) {
+    /* Wait for ready response or command completion */
+    imclient->readytag = imclient->gensym;
+    while (imclient->readytag) {
+      imclient_processoneevent(imclient);
+    }
+    
+    /* stop looping on command completion */
+    if (!imclient->readytxt) break;
+
+    if (isspace(*imclient->readytxt)) {
+	inlen = 0;
+    } else {
+	inlen = imclient_decodebase64(imclient->readytxt);
+    }
+
+    if (inlen == -1) {
+	/* bad base64 string */
+	return replytype_bad;
+    }
+
+    if (inlen == 0 && outlen > 0) {
+	/* we have something from the initial thing to send */
+    } else {
+	/* perform a step */
+	saslresult = SASL_INTERACT;
+	while (saslresult == SASL_INTERACT) {
+	    saslresult=sasl_client_step(imclient->saslconn,
+					imclient->readytxt,
+					inlen, 
+					&client_interact,
+					&out,
+					&outlen);
+	    
+	    if (saslresult == SASL_INTERACT) {
+		/* fill in prompts */
+		fillin_interactions(client_interact, user); 
+	    }
+	}
+    }
+
+    /* send to server */
+    /* Send our reply to the server */
+    if ((saslresult==SASL_OK) || (saslresult==SASL_CONTINUE)) {
+	imclient_writebase64(imclient, out, outlen);
+    } else {
+	return saslresult;
+    }
+
+    if (outlen > 0) { 
+	free(out); 
+    }
+    outlen = 0;
+  }
+
+  imclient->saslcompleted = 1;
+
+  return (result.replytype != replytype_ok);
+}
+
+
+
 #define XX 127
 /*
  * Tables for encoding/decoding base64
@@ -1193,9 +1319,7 @@ static const char index_64[256] = {
  * Decode in-place the base64 data in 'input'.  Returns the length
  * of the decoded data, or -1 if there was an error.
  */
-static int 
-imclient_decodebase64(input)
-char *input;
+static int imclient_decodebase64(char *input)
 {
     int len = 0;
     unsigned char *output = (unsigned char *)input;
@@ -1278,3 +1402,471 @@ unsigned len;
     imclient_write(imclient, buf, buflen);
 }
 
+
+/*************** All these functions help do the starttls; these are copied from imtest.c ********/
+#ifdef HAVE_SSL
+
+static int verify_depth;
+static int verify_error = X509_V_OK;
+static int do_dump = 1;
+
+#define CCERT_BUFSIZ 256
+static char peer_subject[CCERT_BUFSIZ];
+static char peer_issuer[CCERT_BUFSIZ];
+static char peer_CN[CCERT_BUFSIZ];
+static char issuer_CN[CCERT_BUFSIZ];
+static unsigned char md[EVP_MAX_MD_SIZE];
+static char fingerprint[EVP_MAX_MD_SIZE * 3];
+
+char   *tls_peer_CN = "";
+char   *tls_issuer_CN = NULL;
+
+char   *tls_protocol = NULL;
+const char   *tls_cipher_name = NULL;
+int	tls_cipher_usebits = 0;
+int	tls_cipher_algbits = 0;
+
+/*
+  * Set up the cert things on the server side. We do need both the
+  * private key (in key_file) and the cert (in cert_file).
+  * Both files may be identical.
+  *
+  * This function is taken from OpenSSL apps/s_cb.c
+  */
+
+static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
+{
+    if (cert_file != NULL) {
+	if (SSL_CTX_use_certificate_file(ctx, cert_file,
+					 SSL_FILETYPE_PEM) <= 0) {
+	  printf("[ unable to get certificate from '%s' ]\n", cert_file);
+	  return (0);
+	}
+	if (key_file == NULL)
+	    key_file = cert_file;
+	if (SSL_CTX_use_PrivateKey_file(ctx, key_file,
+					SSL_FILETYPE_PEM) <= 0) {
+	  printf("[ unable to get private key from '%s' ]\n", key_file);
+	  return (0);
+	}
+	/* Now we know that a key and cert have been set against
+         * the SSL context */
+	if (!SSL_CTX_check_private_key(ctx)) {
+	  printf("[ Private key does not match the certificate public key ]\n");
+	  return (0);
+	}
+    }
+    return (1);
+}
+
+/* taken from OpenSSL apps/s_cb.c */
+
+static int verify_callback(int ok, X509_STORE_CTX * ctx)
+{
+    char    buf[256];
+    X509   *err_cert;
+    int     err;
+    int     depth;
+
+    err_cert = X509_STORE_CTX_get_current_cert(ctx);
+    err = X509_STORE_CTX_get_error(ctx);
+    depth = X509_STORE_CTX_get_error_depth(ctx);
+
+    X509_NAME_oneline(X509_get_subject_name(err_cert), buf, 256);
+
+    /*    if (verbose==1)
+	  printf("Peer cert verify depth=%d %s\n", depth, buf);*/
+
+    if (!ok) {
+      printf("verify error:num=%d:%s\n", err,
+	     X509_verify_cert_error_string(err));
+	if (verify_depth >= depth) {
+	    ok = 1;
+	    verify_error = X509_V_OK;
+	} else {
+	    ok = 0;
+	    verify_error = X509_V_ERR_CERT_CHAIN_TOO_LONG;
+	}
+    }
+    switch (ctx->error) {
+    case X509_V_ERR_UNABLE_TO_GET_ISSUER_CERT:
+	X509_NAME_oneline(X509_get_issuer_name(ctx->current_cert), buf, 256);
+	printf("issuer= %s\n", buf);
+	break;
+    case X509_V_ERR_CERT_NOT_YET_VALID:
+    case X509_V_ERR_ERROR_IN_CERT_NOT_BEFORE_FIELD:
+      printf("cert not yet valid\n");
+      break;
+    case X509_V_ERR_CERT_HAS_EXPIRED:
+    case X509_V_ERR_ERROR_IN_CERT_NOT_AFTER_FIELD:
+      printf("cert has expired\n");
+      break;
+    }
+
+    /*    if (verbose==1)
+	  printf("verify return:%d\n", ok);*/
+
+    return (ok);
+}
+
+
+/* taken from OpenSSL apps/s_cb.c */
+
+static RSA *tmp_rsa_cb(SSL * s, int export, int keylength)
+{
+    static RSA *rsa_tmp = NULL;
+
+    if (rsa_tmp == NULL) {
+	rsa_tmp = RSA_generate_key(keylength, RSA_F4, NULL, NULL);
+    }
+    return (rsa_tmp);
+}
+
+/* taken from OpenSSL apps/s_cb.c 
+ * tim - this seems to just be giving logging messages
+ */
+
+static void apps_ssl_info_callback(SSL * s, int where, int ret)
+{
+    char   *str;
+    int     w;
+
+    /*    return; */ /* only useful for debugging */
+
+    w = where & ~SSL_ST_MASK;
+
+    if (w & SSL_ST_CONNECT)
+	str = "SSL_connect";
+    else if (w & SSL_ST_ACCEPT)
+	str = "SSL_accept";
+    else
+	str = "undefined";
+
+    if (where & SSL_CB_LOOP) {
+      printf("%s:%s\n", str, SSL_state_string_long(s));
+    } else if (where & SSL_CB_ALERT) {
+	str = (where & SSL_CB_READ) ? "read" : "write";
+	if ((ret & 0xff) != SSL3_AD_CLOSE_NOTIFY)
+	  printf("SSL3 alert %s:%s:%s\n", str,
+		   SSL_alert_type_string_long(ret),
+		   SSL_alert_desc_string_long(ret));
+    } else if (where & SSL_CB_EXIT) {
+	if (ret == 0)
+	    printf("%s:failed in %s\n",
+		     str, SSL_state_string_long(s));
+	else if (ret < 0) {
+	    printf("%s:error in %s %i\n",
+		     str, SSL_state_string_long(s),ret);
+	}
+    }
+}
+
+
+char *var_tls_CAfile="";
+char *var_tls_CApath="";
+ /*
+  * This is the setup routine for the SSL client. 
+  *
+  * The skeleton of this function is taken from OpenSSL apps/s_client.c.
+  */
+
+static int tls_init_clientengine(struct imclient *imclient,
+				 int verifydepth, char *var_tls_cert_file, char *var_tls_key_file)
+{
+    int     off = 0;
+    int     verify_flags = SSL_VERIFY_NONE;
+    char   *CApath;
+    char   *CAfile;
+    char   *c_cert_file;
+    char   *c_key_file;
+
+
+    SSL_load_error_strings();
+    SSLeay_add_ssl_algorithms();
+
+    imclient->tls_ctx = SSL_CTX_new(SSLv23_client_method());
+    if (imclient->tls_ctx == NULL) {
+	return -1;
+    };
+
+    off |= SSL_OP_ALL;		/* Work around all known bugs */
+    SSL_CTX_set_options(imclient->tls_ctx, off);
+    
+    /* debugging   SSL_CTX_set_info_callback(imclient->tls_ctx, apps_ssl_info_callback); */
+
+    if (strlen(var_tls_CAfile) == 0)
+	CAfile = NULL;
+    else
+	CAfile = var_tls_CAfile;
+    if (strlen(var_tls_CApath) == 0)
+	CApath = NULL;
+    else
+	CApath = var_tls_CApath;
+
+    if (CAfile || CApath)
+	if ((!SSL_CTX_load_verify_locations(imclient->tls_ctx, CAfile, CApath)) ||
+	    (!SSL_CTX_set_default_verify_paths(imclient->tls_ctx))) {
+	    printf("[ TLS engine: cannot load CA data ]\n");
+	    return -1;
+	}
+    if (strlen(var_tls_cert_file) == 0)
+	c_cert_file = NULL;
+    else
+	c_cert_file = var_tls_cert_file;
+    if (strlen(var_tls_key_file) == 0)
+	c_key_file = NULL;
+    else
+	c_key_file = var_tls_key_file;
+
+    if (c_cert_file || c_key_file)
+	if (!set_cert_stuff(imclient->tls_ctx, c_cert_file, c_key_file)) {
+	    printf("[ TLS engine: cannot load cert/key data ]\n");
+	    return -1;
+	}
+    SSL_CTX_set_tmp_rsa_callback(imclient->tls_ctx, tmp_rsa_cb);
+
+    verify_depth = verifydepth;
+    SSL_CTX_set_verify(imclient->tls_ctx, verify_flags, verify_callback);
+
+    return 0;
+}
+
+/*
+ * taken from OpenSSL crypto/bio/b_dump.c, modified to save a lot of strcpy
+ * and strcat by Matti Aarnio.
+ */
+
+#define TRUNCATE
+#define DUMP_WIDTH	16
+
+static int tls_dump(const char *s, int len)
+{
+    int     ret = 0;
+    char    buf[160 + 1];
+    char    *ss;
+    int     i;
+    int     j;
+    int     rows;
+    int     trunc;
+    unsigned char ch;
+
+    trunc = 0;
+
+#ifdef TRUNCATE
+    for (; (len > 0) && ((s[len - 1] == ' ') || (s[len - 1] == '\0')); len--)
+	trunc++;
+#endif
+
+    rows = (len / DUMP_WIDTH);
+    if ((rows * DUMP_WIDTH) < len)
+	rows++;
+
+    for (i = 0; i < rows; i++) {
+	buf[0] = '\0';				/* start with empty string */
+	ss = buf;
+
+	sprintf(ss, "%04x ", i * DUMP_WIDTH);
+	ss += strlen(ss);
+	for (j = 0; j < DUMP_WIDTH; j++) {
+	    if (((i * DUMP_WIDTH) + j) >= len) {
+		strcpy(ss, "   ");
+	    } else {
+		ch = ((unsigned char) *((char *) (s) + i * DUMP_WIDTH + j))
+		    & 0xff;
+		sprintf(ss, "%02x[%c]%c", ch, ch, j == 7 ? '|' : ' ');
+		ss += 6;
+	    }
+	}
+	ss += strlen(ss);
+	*ss+= ' ';
+	for (j = 0; j < DUMP_WIDTH; j++) {
+	    if (((i * DUMP_WIDTH) + j) >= len)
+		break;
+	    ch = ((unsigned char) *((char *) (s) + i * DUMP_WIDTH + j)) & 0xff;
+	    *ss+= (((ch >= ' ') && (ch <= '~')) ? ch : '.');
+	    if (j == 7) *ss+= ' ';
+	}
+	*ss = 0;
+	/* 
+	 * if this is the last call then update the ddt_dump thing so that
+         * we will move the selection point in the debug window
+         */
+	printf("%s\n", buf);
+	ret += strlen(buf);
+    }
+#ifdef TRUNCATE
+    if (trunc > 0) {
+	sprintf(buf, "%04x - <SPACES/NULS>\n", len+ trunc);
+	printf("%s\n", buf);
+	ret += strlen(buf);
+    }
+#endif
+    return (ret);
+}
+
+
+/* taken from OpenSSL apps/s_cb.c */
+
+static long bio_dump_cb(BIO * bio, int cmd, const char *argp, int argi,
+			long argl, long ret)
+{
+    if (!do_dump)
+	return (ret);
+
+    if (cmd == (BIO_CB_READ | BIO_CB_RETURN)) {
+	printf("read from %08X [%08lX] (%d bytes => %ld (0x%X))\n", bio, argp,
+		 argi, ret, ret);
+	tls_dump(argp, (int) ret);
+	return (ret);
+    } else if (cmd == (BIO_CB_WRITE | BIO_CB_RETURN)) {
+	printf("write to %08X [%08lX] (%d bytes => %ld (0x%X))\n", bio, argp,
+		 argi, ret, ret);
+	tls_dump(argp, (int) ret);
+    }
+    return (ret);
+}
+
+int tls_start_clienttls(struct imclient *imclient,
+			int *layer, char **authid, int fd)
+{
+    int     sts;
+    int     j;
+    unsigned int n;
+    SSL_SESSION *session;
+    SSL_CIPHER *cipher;
+    X509   *peer;
+
+    if (imclient->tls_conn == NULL) {
+	imclient->tls_conn = (SSL *) SSL_new(imclient->tls_ctx);
+    }
+    if (imclient->tls_conn == NULL) {
+	printf("Could not allocate 'con' with SSL_new()\n");
+	return -1;
+    }
+    SSL_clear(imclient->tls_conn);
+
+    if (!SSL_set_fd(imclient->tls_conn, fd)) {
+      printf("SSL_set_fd failed\n");
+      return -1;
+    }
+
+    /*SSL_set_read_ahead(imclient->tls_conn, 1);*/
+
+    /*
+     * This is the actual handshake routine. It will do all the negotiations
+     * and will check the client cert etc.
+     */
+    SSL_set_connect_state(imclient->tls_conn);
+
+
+    /*
+     * We do have an SSL_set_fd() and now suddenly a BIO_ routine is called?
+     * Well there is a BIO below the SSL routines that is automatically
+     * created for us, so we can use it for debugging purposes.
+     */
+    /*    if (verbose==1) */
+    /*    BIO_set_callback(SSL_get_rbio(imclient->tls_conn), bio_dump_cb);*/
+
+    /* Dump the negotiation for loglevels 3 and 4 */
+
+    if ((sts = SSL_connect(imclient->tls_conn)) <= 0) {
+	printf("[ SSL_connect error %d ]\n", sts); /* xxx get string error? */
+	session = SSL_get_session(imclient->tls_conn);
+	if (session) {
+	    SSL_CTX_remove_session(imclient->tls_ctx, session);
+	    printf("[ SSL session removed ]\n");
+	}
+	if (imclient->tls_conn!=NULL)
+	    SSL_free(imclient->tls_conn);
+	imclient->tls_conn = NULL;
+	return -1;
+    }
+
+    /*
+     * Lets see, whether a peer certificate is availabe and what is
+     * the actual information. We want to save it for later use.
+     */
+    peer = SSL_get_peer_certificate(imclient->tls_conn);
+    if (peer != NULL) {
+	X509_NAME_get_text_by_NID(X509_get_subject_name(peer),
+			  NID_commonName, peer_CN, CCERT_BUFSIZ);
+	tls_peer_CN = peer_CN;
+	X509_NAME_get_text_by_NID(X509_get_issuer_name(peer),
+			  NID_commonName, issuer_CN, CCERT_BUFSIZ);
+	/*	if (verbose==1)
+		printf("subject_CN=%s, issuer_CN=%s\n", peer_CN, issuer_CN);*/
+	tls_issuer_CN = issuer_CN;
+
+    }
+    tls_protocol = SSL_get_version(imclient->tls_conn);
+    cipher = SSL_get_current_cipher(imclient->tls_conn);
+    tls_cipher_name = SSL_CIPHER_get_name(cipher);
+    tls_cipher_usebits = SSL_CIPHER_get_bits(cipher,
+						 &tls_cipher_algbits);
+
+    if (layer!=NULL)
+      *layer = tls_cipher_usebits;
+
+    if (authid!=NULL)
+      *authid = tls_peer_CN;
+
+    /*    printf("TLS connection established: %s with cipher %s (%d/%d bits)\n",
+	   tls_protocol, tls_cipher_name,
+	   tls_cipher_usebits, tls_cipher_algbits);*/
+    return 0;
+}
+
+int imclient_starttls(struct imclient *imclient,
+			     int verifydepth,
+			     char *var_tls_cert_file, 
+			     char *var_tls_key_file,
+			     int *layer)
+{
+  int result;
+  struct authresult theresult;
+
+  sasl_external_properties_t externalprop;
+
+  imclient_send(imclient, tlsresult, (void *)&theresult,
+		"STARTTLS");
+
+  /* Wait for ready response or command completion */
+  imclient->readytag = imclient->gensym;
+  while (imclient->readytag) {
+    imclient_processoneevent(imclient);
+  }
+
+  result=tls_init_clientengine(imclient, 10, var_tls_cert_file, var_tls_key_file);
+  if (result!=0)
+  {
+    printf("[ TLS engine failed ]\n");
+    return 1;
+  } else {
+    result=tls_start_clienttls(imclient, &externalprop.ssf, &externalprop.auth_id, imclient->fd);
+    
+    if (result!=0) {
+      printf("[ TLS negotiation did not succeed ]\n");
+      return 1;
+    }
+  }
+
+  /* turn non-blocking i/o back on */
+
+
+  /* TLS negotiation succeeded */
+
+  imclient->tls_on = 1;
+
+  externalprop.auth_id=""; /* xxx this really should be peer_CN or
+  issuer_CN but I can't figure out which is which at the moment */
+
+  /* tell SASL about the negotiated layer */
+  result=sasl_setprop(imclient->saslconn,
+		      SASL_SSF_EXTERNAL,
+		      &externalprop);
+  
+  if (result!=SASL_OK) return 1;
+
+  return 0;
+}
+#endif /* HAVE_SSL */
