@@ -1,5 +1,5 @@
 /* map_shared.c -- memory-mapping routines.
- $Id: map_shared.c,v 1.12 1998/05/15 21:52:10 neplokh Exp $
+ $Id: map_shared.c,v 1.14 1999/09/30 07:32:00 leg Exp $
  
  #        Copyright 1998 by Carnegie Mellon University
  #
@@ -29,7 +29,8 @@
 #include <syslog.h>
 
 #include "map.h"
-#include "sysexits.h"
+#include "exitcodes.h"
+#include "xmalloc.h"
 
 #define SLOP (8*1024)
 
@@ -54,7 +55,7 @@ const char *mboxname;
 	    syslog(LOG_ERR, "IOERROR: fstating %s file%s%s: %m", name,
 		   mboxname ? " for " : "", mboxname ? mboxname : "");
 	    sprintf(buf, "failed to fstat %s file", name);
-	    fatal(buf, EX_IOERR);
+	    fatal(buf, EC_IOERR);
 	}
 	newlen = sbuf.st_size;
     }
@@ -80,7 +81,7 @@ const char *mboxname;
 	syslog(LOG_ERR, "IOERROR: mapping %s file%s%s: %m", name,
 	       mboxname ? " for " : "", mboxname ? mboxname : "");
 	sprintf(buf, "failed to mmap %s file", name);
-	fatal(buf, EX_IOERR);
+	fatal(buf, EC_IOERR);
     }
     *len = newlen;
 }
