@@ -1,5 +1,5 @@
 /* nonblock_fcntl.c -- Set nonblocking mode using fcntl()
- $Id: nonblock_fcntl.c,v 1.6 1998/05/15 21:52:34 neplokh Exp $
+ $Id: nonblock_fcntl.c,v 1.8 1999/10/30 23:07:40 leg Exp $
  
  #        Copyright 1998 by Carnegie Mellon University
  #
@@ -26,6 +26,8 @@
 #include <sys/file.h>
 #include <fcntl.h>
 
+#include "xmalloc.h"
+#include "exitcodes.h"
 #include "nonblock.h"
 
 #ifndef	FNDELAY
@@ -51,7 +53,7 @@ int mode;
     int flags;
 
     flags = fcntl(fd, F_GETFL, 0);
-    if (flags < 0) fatal("Internal error: fcntl F_GETFL failed");
+    if (flags < 0) fatal("Internal error: fcntl F_GETFL failed", EC_IOERR);
     if (mode) {
 	flags |= NON_BLOCKING_MODE;
     }
