@@ -22,7 +22,7 @@
  *
  */
 /*
- * $Id: prot.c,v 1.34 1998/06/10 22:18:15 tjs Exp $
+ * $Id: prot.c,v 1.45 1999/12/23 18:58:53 leg Exp $
  */
 
 #include <stdio.h>
@@ -34,6 +34,9 @@
 #else
 #include <varargs.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <sys/types.h>
 #include <sys/time.h>
 #include <netinet/in.h>
@@ -54,10 +57,11 @@ int write;
 {
     struct protstream *newstream;
 
-    newstream = (struct protstream *)xmalloc(sizeof(struct protstream));
+    newstream = (struct protstream *) xmalloc(sizeof(struct protstream));
+    newstream->buf = (char *) xmalloc(sizeof(char) * (PROT_BUFSIZE + 4));
+    newstream->buf_size = PROT_BUFSIZE;
     newstream->ptr = newstream->buf;
     newstream->cnt = write ? PROT_BUFSIZE : 0;
-    newstream->leftcnt = 0;
     newstream->maxplain = PROT_BUFSIZE;
     newstream->fd = fd;
     newstream->write = write;
@@ -68,9 +72,16 @@ int write;
     newstream->error = 0;
     newstream->eof = 0;
     newstream->read_timeout = 0;
+    newstream->dontblock = 0;
     newstream->flushonread = 0;
     newstream->readcallback_proc = 0;
     newstream->readcallback_rock = 0;
+    newstream->conn = NULL;
+    newstream->saslssf=0;
+
+#ifdef HAVE_SSL
+    newstream->tls_conn=NULL;
+#endif /* HAVE_SSL */
 
     return newstream;
 }
@@ -81,6 +92,7 @@ int write;
 int prot_free(s)
 struct protstream *s;
 {
+    free(s->buf);
     free((char*)s);
     return 0;
 }
@@ -108,34 +120,68 @@ time_t *ptr;
     return 0;
 }
 
+#ifdef HAVE_SSL
+
 /*
- * Set the protection function for stream 's' to be 'func'.  The opaque
- * object 'state' is passed to 'func' whenever it is called.  If the
- * stream is for writing, 'maxplain' is the maximum number of plaintext
- * bytes that will be given to 'func' at one time.
+ * Turn on TLS for this connection
  */
-int prot_setfunc(s, func, state, maxplain)
-struct protstream *s;
-const char *(*func)();
-void *state;
-int maxplain;
+
+int prot_settls(struct protstream *s, SSL *tlsconn)
 {
-    if (s->write && s->ptr != s->buf) prot_flush(s);
+  s->tls_conn = tlsconn;
 
-    s->func = func;
-    s->state = state;
+  return 0;
+}
 
-    if (s->write) {
-	s->maxplain = maxplain;
-	s->cnt = maxplain;
-    }
-    else if (s->cnt) {
-	s->leftptr = s->ptr;
-	s->leftcnt = s->cnt;
-	s->cnt = 0;
+#endif /* HAVE_SSL */
+
+/*
+ * Turn on SASL for this connection
+ */
+
+int prot_setsasl(s, conn)
+struct protstream *s;
+sasl_conn_t *conn;
+{
+  int *ssfp;
+  int result;
+
+  /* flush first if need be */
+  if (s->write && s->ptr != s->buf) prot_flush(s);
+
+  s->conn=conn;
+
+  result = sasl_getprop(conn, SASL_SSF, (void **) &ssfp);
+  if (result != SASL_OK)
+    return 1;
+
+  s->saslssf = *ssfp;
+
+  if (s->write) {
+    int result;
+    int *maxp, max;
+
+    /* ask SASL for layer max */
+    result = sasl_getprop(conn, SASL_MAXOUTBUF, (void **) &maxp);
+    max = *maxp;
+    if (result != SASL_OK)
+      return 1;
+
+    if (max == 0 || max > PROT_BUFSIZE) {
+	/* max = 0 means unlimited, and we can't go bigger */
+	max = PROT_BUFSIZE;
     }
+    
+    max-=50; /* account for extra foo incurred from layers */
 
-    return 0;
+    s->maxplain=max;
+    s->cnt=max;
+  }
+  else if (s->cnt) {  /* XXX what does this do? */
+    s->cnt = 0;
+  }
+
+  return 0;
 }
 
 /*
@@ -199,7 +245,7 @@ struct protstream *s;
 	s->error = strerror(errno);
 	return EOF;
     }
-    s->cnt = s->leftcnt = 0;
+    s->cnt = 0;
     s->error = 0;
     s->eof = 0;
     return 0;
@@ -213,82 +259,131 @@ int
 prot_fill(s)
 struct protstream *s;
 {
-    int n, cnt = 0;
-    unsigned inputlen = 0;
+    int n;
     char *ptr;
-    const char *err;
     int left;
     int r;
     struct timeval timeout;
     fd_set rfds;
-    int haveinput;
-    
+    int haveinput; 
+   
     if (s->eof || s->error) return EOF;
 
     do {
-	if (s->leftcnt) {
-	    /* Crypttext left over from last fill, process it */
-	    n = s->leftcnt;
-	    memmove(s->buf, s->leftptr, n);
-	    s->leftcnt = 0;
-	}
-	else {
-	    haveinput = 0;
-	    if (s->readcallback_proc ||
-		(s->flushonread && s->flushonread->ptr != s->flushonread->buf)) {
-		timeout.tv_sec = timeout.tv_usec = 0;
-		FD_ZERO(&rfds);
-		FD_SET(s->fd, &rfds);
-		if (select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
-			   &timeout) <= 0) {
-		    if (s->readcallback_proc) {
-			(*s->readcallback_proc)(s, s->readcallback_rock);
-			s->readcallback_proc = 0;
-			s->readcallback_rock = 0;
-		    }
-		    if (s->flushonread) prot_flush(s->flushonread);
-		}
-		else {
-		    haveinput = 1;
-		}
-	    }
 
-	    if (!haveinput && s->read_timeout) {
-		timeout.tv_sec = s->read_timeout;
-		timeout.tv_usec = 0;
-		FD_ZERO(&rfds);
-		FD_SET(s->fd, &rfds);
-		do {
-		    r = select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
-			       &timeout);
-		} while (r == -1 && errno == EINTR);
-		if (r == 0) {
+	/* wait until get input */
+	haveinput = 0;
+	if (s->readcallback_proc ||
+	    (s->flushonread && s->flushonread->ptr != s->flushonread->buf)) {
+	    timeout.tv_sec = timeout.tv_usec = 0;
+	    FD_ZERO(&rfds);
+	    FD_SET(s->fd, &rfds);
+	    if (select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
+		       &timeout) <= 0) {
+		if (s->readcallback_proc) {
+		    (*s->readcallback_proc)(s, s->readcallback_rock);
+		    s->readcallback_proc = 0;
+		    s->readcallback_rock = 0;
+		}
+		if (s->flushonread) prot_flush(s->flushonread);
+	    }
+	    else {
+		haveinput = 1;
+	    }
+	}
+
+	if (!haveinput && (s->read_timeout || s->dontblock)) {
+	    timeout.tv_sec = s->read_timeout;
+	    timeout.tv_usec = 0;
+	    FD_ZERO(&rfds);
+	    FD_SET(s->fd, &rfds);
+	    do {
+		r = select(s->fd + 1, &rfds, (fd_set *)0, (fd_set *)0,
+			   &timeout);
+	    } while (r == -1 && errno == EINTR);
+	    if (r == 0) {
+		if (!s->dontblock) {
 		    s->error = "idle for too long";
 		    return EOF;
+		} else {
+		    errno = EAGAIN;
+		    return EOF;
 		}
 	    }
-	    do {
-		n = read(s->fd, s->buf+cnt, sizeof(s->buf)-cnt);
-	    } while (n == -1 && errno == EINTR);
 	}
-    
+	
+	do {
+#ifdef HAVE_SSL	  
+
+	  /* just do a SSL read instead if we're under a tls layer */
+	  if (s->tls_conn!=NULL)
+	  {
+	    n = SSL_read(s->tls_conn, s->buf, PROT_BUFSIZE);
+	  } else {
+	    n = read(s->fd, s->buf, PROT_BUFSIZE);
+	  }
+
+#else  /* HAVE_SSL */
+	  n = read(s->fd, s->buf, PROT_BUFSIZE);
+#endif /* HAVE_SSL */
+
+
+	    
+	} while (n == -1 && errno == EINTR);
+	
+	
 	if (n <= 0) {
 	    if (n) s->error = strerror(errno);
 	    else s->eof = 1;
 	    return EOF;
 	}
-
-	if (!s->func) {
+	
+	if (s->saslssf) { /* decode it */
+	    int result;
+	    char *out;
+	    unsigned outlen;
+	    
+	    /* Decode the input token */
+	    result = sasl_decode(s->conn, (const char *) s->buf, n, 
+				 &out, &outlen);
+	    
+	    if (result != SASL_OK) {
+		snprintf(s->buf, 200, "Decoding error: %s (%i)",
+			 sasl_errstring(result, NULL, NULL), result);
+		s->error = s->buf;
+		return EOF;
+	    }
+	    
+	    if (outlen > 0) {
+		if (outlen > s->buf_size) {
+		    s->buf = (char *) xrealloc(s->buf, 
+					       sizeof(char) * (outlen + 4));
+		    s->buf_size = outlen;
+		}
+		memcpy(s->buf, out, outlen);
+		s->ptr = s->buf + 1;
+		s->cnt = outlen;
+		free(out);
+	    } else {		/* didn't decode anything */
+		s->cnt = 0;
+	    }
+	    
+	} else {
 	    /* No protection function, just use the raw data */
-	    s->cnt = n-1;
 	    s->ptr = s->buf+1;
+	    s->cnt = n;
+	}
+	
+	  
+
+	if (s->cnt > 0) {
 	    if (s->logfd != -1) {
 		time_t newtime;
 		char timebuf[20];
 
 		if (s->log_timeptr) {
 		    time(&newtime);
-		    sprintf(timebuf, "<%d<", newtime - *s->log_timeptr);
+		    sprintf(timebuf, "<%ld<", newtime - *s->log_timeptr);
 		    write(s->logfd, timebuf, strlen(timebuf));
 		    *s->log_timeptr = newtime;
 		}
@@ -306,61 +401,12 @@ struct protstream *s;
 		    }
 		} while (left);
 	    }
+	    s->cnt--;		/* we return the first char */
 	    return *s->buf;
 	}
-	cnt += n;
-	/* First 4 bytes contain length of crypttext token */
-	if (!inputlen && cnt >= 4) {
-	    inputlen = ntohl(*(int *)s->buf);
-	    if (inputlen > sizeof(s->buf) - 4) {
-		s->error = "Input crypttext token too long";
-		return EOF;
-	    }
-	}
-    } while (cnt < 4 || cnt-4 < inputlen);
 
-    /* Decode the input token */
-    err = s->func(s->state, s->buf+4, inputlen, &s->ptr, &s->cnt);
-    if (err) {
-	strcpy(s->buf, "Decoding error: ");
-	strcat(s->buf, err);
-	s->error = s->buf;
-	return EOF;
-    }
+    } while (1);
 
-    /* Save any left-over crypttext data for next time */
-    if (cnt > inputlen + 4) {
-	s->leftptr = s->buf + inputlen + 4;
-	s->leftcnt = cnt - (inputlen + 4);
-    }
-
-    if (s->logfd != -1) {
-	time_t newtime;
-	char timebuf[20];
-
-	if (s->log_timeptr) {
-	    time(&newtime);
-	    sprintf(timebuf, "<%d<", newtime - *s->log_timeptr);
-	    write(s->logfd, timebuf, strlen(timebuf));
-	    *s->log_timeptr = newtime;
-	}
-
-	left = s->cnt;
-	ptr = s->ptr;
-	do {
-	    n = write(s->logfd, ptr, left);
-	    if (n == -1 && errno != EINTR) {
-		break;
-	    }
-	    if (n > 0) {
-		ptr += n;
-		left -= n;
-	    }
-	} while (left);
-    }
-
-    s->cnt--;
-    return *s->ptr++;
 }
 
 /*
@@ -369,10 +415,10 @@ struct protstream *s;
 int prot_flush(s)
 struct protstream *s;
 {
-    unsigned char outputbuf[PROT_BUFSIZE+4];
-    unsigned char *ptr = s->buf;
+    char *ptr = s->buf;
     int left = s->ptr - s->buf;
     int n;
+    char *foo;
 
     if (s->eof || s->error) {
 	s->ptr = s->buf;
@@ -387,7 +433,7 @@ struct protstream *s;
 
 	if (s->log_timeptr) {
 	    time(&newtime);
-	    sprintf(timebuf, ">%d>", newtime - *s->log_timeptr);
+	    sprintf(timebuf, ">%ld>", newtime - *s->log_timeptr);
 	    write(s->logfd, timebuf, strlen(timebuf));
 	}
 
@@ -405,32 +451,54 @@ struct protstream *s;
 	ptr = s->buf;
     }
 
-    if (s->func) {
-	/* Encode the data */
-	if (s->func(s->state, ptr, left, outputbuf+4, &left)) {
-	    s->error = "Encoding error";
-	    if (s->log_timeptr) time(s->log_timeptr);
-	    return EOF;
-	}
-	*(int *)outputbuf = htonl(left);
-	ptr = outputbuf;
-	left += 4;
+
+
+    if (s->saslssf!=0) {
+      /* Encode the data */  /* xxx handle left */
+      unsigned int outlen;
+      int result;
+
+      result=sasl_encode(s->conn, ptr, left, &foo, &outlen);
+      if (result!=SASL_OK)
+      {
+	s->error = "Encoding error";
+	if (s->log_timeptr) time(s->log_timeptr);
+	return EOF;
+      }
+
+      ptr=foo;
+      left=outlen;
     }
 
     /* Write out the data */
     do {
+#ifdef HAVE_SSL
+      if (s->tls_conn!=NULL)
+      {
+        n = SSL_write(s->tls_conn, ptr, left);
+      } else {
 	n = write(s->fd, ptr, left);
+      }
+#else  /* HAVE_SSL */
+	n = write(s->fd, ptr, left);
+#endif /* HAVE_SSL */
 	if (n == -1 && errno != EINTR) {
 	    s->error = strerror(errno);
 	    if (s->log_timeptr) time(s->log_timeptr);
 	    return EOF;
 	}
+
 	if (n > 0) {
 	    ptr += n;
 	    left -= n;
 	}
     } while (left);
 
+    /* sasl_encode did a malloc so we need to free for it */
+    if (s->saslssf!=0) { 
+      free(foo);
+    }
+
     /* Reset the output buffer */
     s->ptr = s->buf;
     s->cnt = s->maxplain;
@@ -489,7 +557,7 @@ va_dcl
     fmt = va_arg(pvar, char *);
 #endif
 
-    while (percent = strchr(fmt, '%')) {
+    while ((percent = strchr(fmt, '%')) != 0) {
 	prot_write(s, fmt, percent-fmt);
 	switch (*++percent) {
 	case '%':
