@@ -1,5 +1,5 @@
 /* prot.h -- stdio-like module that handles IMAP protection mechanisms
- $Id: prot.h,v 1.13 1998/05/15 21:52:52 neplokh Exp $
+   $Id: prot.h,v 1.22 1999/12/23 18:58:53 leg Exp $
  
  #        Copyright 1998 by Carnegie Mellon University
  #
@@ -27,6 +27,14 @@
 #define INCLUDED_PROT_H
 
 #include <time.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+#include <sasl.h>
+
+#ifdef HAVE_SSL
+#include <openssl/ssl.h>
+#endif /* HAVE_SSL */
 
 #ifndef P
 #ifdef __STDC__
@@ -43,24 +51,30 @@ struct protstream;
 typedef void prot_readcallback_t P((struct protstream *s, void *rock));
 
 struct protstream {
-    unsigned char buf[PROT_BUFSIZE+4];
     unsigned char *ptr;
     int cnt;
-    unsigned char *leftptr;
-    unsigned char leftcnt;
     int fd;
     int write;
     int logfd;
+    sasl_conn_t *conn;
+    int saslssf;
     time_t *log_timeptr;
     const char *(*func)();
     void *state;
     int maxplain;
     const char *error;
     int eof;
+    int dontblock;
     int read_timeout;
     struct protstream *flushonread;
     prot_readcallback_t *readcallback_proc;
     void *readcallback_rock;
+    int buf_size;
+    unsigned char *buf;
+
+#ifdef HAVE_SSL
+    SSL *tls_conn;
+#endif /* HAVE_SSL */
 };
 
 #define prot_getc(s) ((s)->cnt-- > 0 ? (int)*(s)->ptr++ : prot_fill(s))
@@ -71,8 +85,10 @@ extern struct protstream *prot_new P((int fd, int write));
 extern int prot_free P((struct protstream *s));
 extern int prot_setlog P((struct protstream *s, int fd));
 extern int prot_setlogtime P((struct protstream *s, time_t *ptr));
-extern int prot_setfunc P((struct protstream *s,
-			   const char *(*func)(), void *state, int maxplain));
+extern int prot_setsasl P((struct protstream *s, sasl_conn_t *conn));
+#ifdef HAVE_SSL
+extern int prot_settls P((struct protstream *s, SSL *tlsconn));
+#endif /* HAVE_SSL */
 extern int prot_settimeout P((struct protstream *s, int timeout));
 extern int prot_setflushonread P((struct protstream *s,
 				  struct protstream *flushs));
